//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Agent.h: interface for the Agent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AGENT_H__44214124_3A7D_11D4_9F88_0000E87A7A07__INCLUDED_)
#define AFX_AGENT_H__44214124_3A7D_11D4_9F88_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Analysis.h"
//#include "Message.h"
#include "Settings.h"
#include <string>
#include <list>

class Message;
class AgentType;

using namespace std;



class Agent  
{
public:
	Agent(int zoneType = 0);
	Agent(const Agent& original);				// Copy constructor
	virtual ~Agent();

	virtual bool		perZoneType() = 0;		// Is this agent type separated by Zone Type or not?
	virtual bool		hasInterface() = 0;		// Does this agent have an editing interface?

	virtual	int			startingDeployment() = 0;	// How many of these to deploy initially
	virtual int			startingWeight() = 0;		// What is the starting weight of this agent type?

	virtual Agent*		newOne(int zoneType = 0) = 0;

	virtual double		getImpression(bool& valid);		// Returns current impression, valid indicates whether or not to ignore it
	virtual void		receiveMessage(Message* theMessage);
	virtual void		sendMessage(Message* theMessage);
	virtual void		copyProperties(Agent* original) = 0;

	virtual bool		step(int stepNum) = 0;		// This is the main execution loop of the agent, return false if finished

	virtual string		getLocalType();				// Use this function to return the type (string) of your agent subclass

	bool				isFinished() {return finished;};
	virtual void		placeDown();	// This places an agent in a new representation; override if necessary but call this one

	virtual	void		doInterface() {};

	Agent*				getID()	{return this;};
	list<string>*		getFullType() {return &fullType;};

	int					getZoneType() {return _zoneType;};
	void				setZoneType(int zoneType) {_zoneType = zoneType;};

	bool				isOfType(string testType);

	virtual void		setType(string typeName) {};	// May or may not be possible. You decide.

	AgentType*			makeType();

	void				setAgentType(AgentType* myType);   // Called automatically. Don't call this.

	virtual void		reportSettings() {};		// Add settings to Settings		NOTE: Agents are not Settables!
	virtual void		receiveSettings() {};		// Get settings from Settings		NOTE: Agents are not Settables!

protected:
	double	impression;	// This stores an agent's current impression of a representation (should be 0-1 and converge to final impression

protected:

	virtual void		loadAnalyses();

	bool			finished;
	list<string>	fullType;
	int				_zoneType;
	bool			_valid;
	AgentType*		_myType;

};

typedef Agent* AgentID;

#endif // !defined(AFX_AGENT_H__44214124_3A7D_11D4_9F88_0000E87A7A07__INCLUDED_)
