//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentLocalBoxDialog.cpp : implementation file
//

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentLocalBoxDialog.h"
#include "ZoneTypes.h"
#include "AgentControl.h"
#include "AgentType.h"
#include <iostream>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AgentLocalBoxDialog dialog


AgentLocalBoxDialog::AgentLocalBoxDialog(CWnd* pParent /*=NULL*/)
	: CDialog(AgentLocalBoxDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(AgentLocalBoxDialog)
	m_bIntersect = FALSE;
	m_intersectionWeight = 0;
	//}}AFX_DATA_INIT
}


void AgentLocalBoxDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(AgentLocalBoxDialog)
	DDX_Check(pDX, IDC_INTERSECT, m_bIntersect);
	DDX_Text(pDX, IDC_INTERSECTION_WEIGHT, m_intersectionWeight);
	DDV_MinMaxInt(pDX, m_intersectionWeight, -100, 100);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(AgentLocalBoxDialog, CDialog)
	//{{AFX_MSG_MAP(AgentLocalBoxDialog)
	ON_LBN_SELCHANGE(IDC_BINDING_LIST, OnSelchangeBindingList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// AgentLocalBoxDialog message handlers

BOOL AgentLocalBoxDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	((CStatic*)GetDlgItem(IDC_ZONE_TYPE))->SetWindowText(("Bounding Box Settings for ZoneType: " + _zoneName).c_str());
	InitZoneList();
	
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void AgentLocalBoxDialog::OnSelchangeBindingList() 
{
	_zoneSelected = ((CListBox*)GetDlgItem(IDC_BINDING_LIST))->GetCurSel();
	/*if(_zoneSelected >= _zoneType)	{
		m_intersectionWeight = _intersectionWeights[_zoneSelected+1];
	}
	else	{*/
		m_intersectionWeight = _intersectionWeights[_zoneSelected];
	//}
}



void AgentLocalBoxDialog::InitZoneList()
{
	if(ZoneTypes::zoneTypes->getNumZoneTypes() > 0)	{
		CListBox* zoneList = (CListBox*)GetDlgItem(IDC_BINDING_LIST);
		zoneList->ResetContent();
		string zoneName;
		for(int i=0; i < ZoneTypes::zoneTypes->getNumZoneTypes(); i++)	{
			zoneName = ZoneTypes::zoneTypes->getZoneName(i);
			//if(_zoneName!=zoneName)
				zoneList->AddString(zoneName.c_str());
		}
		zoneList->SetCurSel(0);
		_zoneSelected = 0;
		/*if(_zoneSelected >= _zoneType)	{
			m_intersectionWeight = _intersectionWeights[_zoneSelected+1];
		}
		else	{*/
			m_intersectionWeight = _intersectionWeights[_zoneSelected];
		//}
		UpdateData(false);
	}
	else
		_zoneSelected = -1;
}


void AgentLocalBoxDialog::setIntersectionWeight(const vector<int> intersectionWeights)
{
	_intersectionWeights.clear();
	_intersectionWeights.resize(intersectionWeights.size());
	for(int i=0; i < _intersectionWeights.size(); i++)
		_intersectionWeights[i] = intersectionWeights[i];
}


void AgentLocalBoxDialog::setIntersectionType(const vector<bool> intersectionTypes)
{
	_IntersectionTypes.clear();
	_IntersectionTypes.resize(intersectionTypes.size());
	for(int i=0; i < intersectionTypes.size(); i++)
		_IntersectionTypes[i] = intersectionTypes[i];
}

void AgentLocalBoxDialog::getIntersectionWeight(vector<int>& intersectionWeights)
{
	intersectionWeights.clear();
	intersectionWeights.resize(_intersectionWeights.size());
	for(int i=0; i < _intersectionWeights.size(); i++)
		intersectionWeights[i] = _intersectionWeights[i];
}


void AgentLocalBoxDialog::getIntersectionType(vector<bool>& intersectionTypes)
{
	intersectionTypes.clear();
	intersectionTypes.resize(_IntersectionTypes.size());
	for(int i=0; i < _IntersectionTypes.size(); i++)
		intersectionTypes[i] = _IntersectionTypes[i];
}

