//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentLocalizedBoundingBox.cpp: implementation of the AgentLocalizedBoundingBox class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentLocalizedBoundingBox.h"
#include "AgentLocalBoxDialog.h"
#include "AgentControl.h"
#include "AgentType.h"
#include "ZoneTypes.h"
#include "Settings.h"
#include <math.h>
#include <iostream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentLocalizedBoundingBox::AgentLocalizedBoundingBox(int zoneType /*= 0*/)
:Agent(zoneType)
{
	fullType.push_front(getLocalType());
	receiveSettings();

	loadAnalyses();
}

AgentLocalizedBoundingBox::~AgentLocalizedBoundingBox()
{
	
}
 
string    AgentLocalizedBoundingBox::getLocalType()
{
	 return "AgentLocalBoundingBox";
 
}

void	AgentLocalizedBoundingBox::loadAnalyses() 
{
	localAnalysis = (AnalysisLocalizedBoundingBox*)(AgentControl::getCurrentDesign()->getAnalysis("AnalysisLocalizedBoundingBox"));
}

bool	AgentLocalizedBoundingBox::step(int stepNum) 
{
	vector<int>		numberOfIntersections, numberOfSurfaces;
	vector<int>		zoneOfOtherCurve;
	numberOfIntersections.resize(ZoneTypes::zoneTypes->getNumZoneTypes());
	numberOfSurfaces.resize(ZoneTypes::zoneTypes->getNumZoneTypes());

	localAnalysis->getBoundaryRelations(numberOfIntersections, numberOfSurfaces, _zoneType);
	//localAnalysis->getBoundaryRelations(bounds, zoneOfOtherCurve, _BindingType, _zoneType);
	
	impression = 0;
	int size = numberOfIntersections.size();
	if(size > 0)	{
		for(int i=0; i<size; i++)	{
			if(!_IntersectionType[i])	
				impression += numberOfIntersections[i]*_IntersectionWeight[i];
			else if(numberOfIntersections[i]==0)
				impression += _IntersectionWeight[i]*2;
		}
		_valid = true;
	}
	else
		_valid = false;
	return false;
}

Agent*	AgentLocalizedBoundingBox::newOne(int zoneType /*= 0*/)
{
	return new AgentLocalizedBoundingBox(zoneType);
}

void	AgentLocalizedBoundingBox::copyProperties(Agent* original)
{
	AgentLocalizedBoundingBox* realOriginal = (AgentLocalizedBoundingBox*)original;

	_IntersectionWeight = realOriginal->_IntersectionWeight;
	_IntersectionType = realOriginal->_IntersectionType;
}

void	AgentLocalizedBoundingBox::doInterface()
{
	AgentLocalBoxDialog	modifyDialog;
	modifyDialog.setIntersectionWeight(_IntersectionWeight);
	modifyDialog.setIntersectionType(_IntersectionType);
	
	modifyDialog.setZoneName(ZoneTypes::zoneTypes->getZoneName(_zoneType));
	modifyDialog.setZoneType(_zoneType);

	if (modifyDialog.DoModal() == IDOK) {
		modifyDialog.getIntersectionWeight(_IntersectionWeight);
		modifyDialog.getIntersectionType(_IntersectionType);
	}
}

void		AgentLocalizedBoundingBox::reportSettings()
{
	for(int i=0; i < ZoneTypes::zoneTypes->getNumZoneTypes(); i++)	{
		Settings::settings->setSetting(getLocalType(), "IntersectionWeight", _IntersectionWeight[i], i);
		Settings::settings->setSetting(getLocalType(), "IntersectionType", (int)_IntersectionType[i], i);
	}
}

void		AgentLocalizedBoundingBox::receiveSettings()
{
	if(! Settings::zoneTypesInitialized)
		return;

	initWeightsMatrices();
	
	int tempInt;
	for(int i=0; i < ZoneTypes::zoneTypes->getNumZoneTypes(); i++)	{
		if (Settings::settings->getSetting(getLocalType(), "IntersectionWeight", tempInt, i)) {
			_IntersectionWeight[i] = tempInt;
		}
		else _IntersectionWeight[i] = INTERSECTION_WEIGHT;
		
		if (Settings::settings->getSetting(getLocalType(), "IntersectionType", tempInt, i)) {
			_IntersectionType[i] = (bool)tempInt;
		}
		else _IntersectionType[i] = false;
	}
}

bool AgentLocalizedBoundingBox::initWeightsMatrices()
{
	int nZoneTypes = ZoneTypes::zoneTypes->getNumZoneTypes();
	if( nZoneTypes < 1)
		return false;
	_IntersectionWeight.resize(nZoneTypes);
	_IntersectionType.resize(nZoneTypes);
}
