//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentNumShapes.cpp: implementation of the AgentNumShapes class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentNumShapes.h"
#include "AgentControl.h"
#include "ZoneTypes.h"
#include "AgentNumShapesDialog.h"
#include "AgentType.h"
#include <math.h>
#include <iostream>
#include <sstream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentNumShapes::AgentNumShapes(int zoneType)
:Agent(zoneType)
{
	fullType.push_front(getLocalType());
	receiveSettings();
	loadAnalyses();
}

AgentNumShapes::~AgentNumShapes()
{

}

void	AgentNumShapes::loadAnalyses() {
	countAnalysis = (AnalysisCountShapes*)(AgentControl::getCurrentDesign()->getAnalysis("AnalysisCountShapes"));
}

bool	AgentNumShapes::step(int stepNum) {

	int count;
	int total;
	

	countAnalysis->getCount(count, total, _zoneType);

	if (total <= 0) {
		_valid = false;
		impression = 0;
		return false;
	}

	_valid = true;

	int kinds = ZoneTypes::zoneTypes->getNumZoneTypes();

	impression = fabs(_desired - ((100.0 * count) / total));

	/*double temp = pow(kinds, (_desired / 50.0) - 1.0);
	temp *= _desired;
	if (_desired != 0) temp /= _desired;

	double expected = total / (kinds * temp);

	impression = 100.0 * fabs((expected - count) / total);*/

	return false;	// Agent done.
}

Agent*	AgentNumShapes::newOne(int zoneType /*= 0*/)
{
	 return new AgentNumShapes(zoneType);
}

void	AgentNumShapes::copyProperties(Agent* original)
{
	AgentNumShapes* realOriginal = (AgentNumShapes*)original;

	_desired = realOriginal->_desired;
}

void	AgentNumShapes::doInterface()
{
	AgentNumShapesDialog	modifyDialog;

	modifyDialog.setProportion(_desired);
	modifyDialog.setZoneName(ZoneTypes::zoneTypes->getZoneName(_zoneType));

	modifyDialog.setZoneName(ZoneTypes::zoneTypes->getZoneName(_zoneType));

	if (modifyDialog.DoModal() == IDOK) {
		_desired = modifyDialog.getProportion();

		if (modifyDialog.getCopyAcross())
			_myType->copyAgentSettingsAcrossAllAgents(this);
	}
}


void		AgentNumShapes::reportSettings()
{
	Settings::settings->setSetting(getLocalType(), "Desired", _desired, _zoneType);
}

void		AgentNumShapes::receiveSettings()
{
	if (! Settings::settings->getSetting(getLocalType(), "Desired", _desired, _zoneType))
		_desired = NS_DESIRED;
}