//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentOverallDesignHeight.cpp: implementation of the AgentOverallDesignHeight class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentOverallDesignHeight.h"
#include "AgentControl.h"
#include "ZoneTypes.h"
#include "AgentOverallDesignHeightDialog.h"
#include "AgentType.h"
#include <math.h>
#include <iostream>
#include <sstream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentOverallDesignHeight::AgentOverallDesignHeight()
{

}

AgentOverallDesignHeight::~AgentOverallDesignHeight()
{

}

void	AgentOverallDesignHeight::loadAnalyses() {
	heightAnalysis = (AnalysisOverallDesignHeight*)(AgentControl::getCurrentDesign()->getAnalysis("AnalysisOverallDesignHeight"));
}

bool	AgentOverallDesignHeight::step(int stepNum) {

	double zmin, zmax, height;

	heightAnalysis->getDesignHeight(zmin, zmax, height);

	if (height < 0) {
		_valid = false;
		impression = 0;
		return false;
	}

	_valid = true;

	impression = fabs(_desiredHeight - height);

	return false;	// Agent done.
}

Agent*	AgentOverallDesignHeight::newOne(int zoneType /*= 0*/)
{
	 return new AgentOverallDesignHeight();
}

void	AgentOverallDesignHeight::copyProperties(Agent* original)
{
	AgentOverallDesignHeight* realOriginal = (AgentOverallDesignHeight*)original;

	_desiredHeight = realOriginal->_desiredHeight;
}

void	AgentOverallDesignHeight::doInterface()
{
	AgentOverallDesignHeightDialog	modifyDialog;

	modifyDialog.setDesiredHeight(_desiredHeight);
	
	if (modifyDialog.DoModal() == IDOK)
		_desiredHeight = modifyDialog.getDesiredHeight();

}


void		AgentOverallDesignHeight::reportSettings()
{
	Settings::settings->setSetting(getLocalType(), "DesiredHeight", _desiredHeight);
}

void		AgentOverallDesignHeight::receiveSettings()
{
	if (! Settings::settings->getSetting(getLocalType(), "DesiredHeight", _desiredHeight))
		_desiredHeight = HEIGHT_DESIRED;
}