//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentShapeSize.cpp: implementation of the AgentShapeSize class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentShapeSize.h"
#include "AgentShapeSizeDialog.h"
#include "AgentControl.h"
#include "AgentType.h"
#include "ZoneTypes.h"
#include <math.h>
#include <iostream>
#include <sstream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentShapeSize::AgentShapeSize(int zoneType /* = 0*/)
:Agent(zoneType)
{
/*	_minX = SS_MIN_X;
	_minY = SS_MIN_Y;
	_minZ = SS_MIN_Z;
	_maxX = SS_MAX_X;
	_maxY = SS_MAX_Y;
	_maxZ = SS_MAX_Z;

	_XWeight = SS_XWEIGHT;
	_YWeight = SS_YWEIGHT;
	_ZWeight = SS_ZWEIGHT;*/

	fullType.push_front(getLocalType());
	receiveSettings();
	loadAnalyses();
}

AgentShapeSize::~AgentShapeSize()
{

}

void	AgentShapeSize::loadAnalyses() {
	sizeAnalysis = (AnalysisShapeSize*)(AgentControl::getCurrentDesign()->getAnalysis("AnalysisShapeSize"));
}

bool	AgentShapeSize::step(int stepNum) {

	bool any;
	double testMinX;
	double testMinY;
	double testMinZ;
	double testMaxX;
	double testMaxY;
	double testMaxZ;

	sizeAnalysis->getSize(any, testMinX, testMinY, testMinZ, testMaxX, testMaxY, testMaxZ, _zoneType);
	if (any) {
		if (testMinX < _minX) impression += (_minX - testMinX) * _XWeight / 2.0;
		if (testMinY < _minY) impression += (_minY - testMinY) * _YWeight / 2.0;
		if (testMinZ < _minZ) impression += (_minZ - testMinZ) * _ZWeight / 2.0;
		if (testMaxX > _maxX) impression += (testMaxX - _maxX) * _XWeight / 2.0;
		if (testMaxY > _maxY) impression += (testMaxY - _maxY) * _YWeight / 2.0;
		if (testMaxZ > _maxZ) impression += (testMaxZ - _maxZ) * _ZWeight / 2.0;

		if (_XWeight + _YWeight + _ZWeight == 0.0) impression = 0;
		else impression /= (_XWeight + _YWeight + _ZWeight);

		_valid = true;
	
	}
	else {
		_valid = false;
		impression = 0;
	}


	return false;
}

Agent*	AgentShapeSize::newOne(int zoneType /*= 0*/)
{
	 return new AgentShapeSize(zoneType);
}

void	AgentShapeSize::copyProperties(Agent* original)
{
	AgentShapeSize* realOriginal = (AgentShapeSize*)original;

	_minX = realOriginal->_minX;
	_minY = realOriginal->_minY;
	_minZ = realOriginal->_minZ;
	_maxX = realOriginal->_minX;
	_maxY = realOriginal->_maxY;
	_maxZ = realOriginal->_maxZ;

	_XWeight = realOriginal->_XWeight;
	_YWeight = realOriginal->_YWeight;
	_ZWeight = realOriginal->_ZWeight;
}

void	AgentShapeSize::doInterface()
{
	AgentShapeSizeDialog	modifyDialog;
	modifyDialog.setXMin(_minX);
	modifyDialog.setXMax(_maxX);
	modifyDialog.setYMin(_minY);
	modifyDialog.setYMax(_maxY);
	modifyDialog.setZMin(_minZ);
	modifyDialog.setZMax(_maxZ);
	modifyDialog.setXWeight(_XWeight);
	modifyDialog.setYWeight(_YWeight);
	modifyDialog.setZWeight(_ZWeight);

	modifyDialog.setZoneName(ZoneTypes::zoneTypes->getZoneName(_zoneType));

	modifyDialog.setZoneName(ZoneTypes::zoneTypes->getZoneName(_zoneType));

	if (modifyDialog.DoModal() == IDOK) {
		_minX = modifyDialog.getXMin();
		_maxX = modifyDialog.getXMax();
		_minY = modifyDialog.getYMin();
		_maxY = modifyDialog.getYMax();
		_minZ = modifyDialog.getZMin();
		_maxZ = modifyDialog.getZMax();
		_XWeight = modifyDialog.getXWeight();
		_YWeight = modifyDialog.getYWeight();
		_ZWeight = modifyDialog.getZWeight();

		if (modifyDialog.getCopyAcross())
			_myType->copyAgentSettingsAcrossAllAgents(this);
	}
}


void		AgentShapeSize::reportSettings()
{
	Settings::settings->setSetting(getLocalType(), "MinX", _minX, _zoneType);
	Settings::settings->setSetting(getLocalType(), "MinY", _minY, _zoneType);
	Settings::settings->setSetting(getLocalType(), "MinZ", _minZ, _zoneType);
	Settings::settings->setSetting(getLocalType(), "MaxX", _maxX, _zoneType);
	Settings::settings->setSetting(getLocalType(), "MaxY", _maxY, _zoneType);
	Settings::settings->setSetting(getLocalType(), "MaxZ", _maxZ, _zoneType);
	Settings::settings->setSetting(getLocalType(), "WeightX", _XWeight, _zoneType);
	Settings::settings->setSetting(getLocalType(), "WeightY", _YWeight, _zoneType);
	Settings::settings->setSetting(getLocalType(), "WeightZ", _ZWeight, _zoneType);
}

void		AgentShapeSize::receiveSettings()
{
	int tempInt;
	double tempDouble;

	if (Settings::settings->getSetting(getLocalType(), "MinX", tempDouble, _zoneType)) {
		_minX = tempDouble;
	}
	else _minX = SS_MIN_X;

	if (Settings::settings->getSetting(getLocalType(), "MinY", tempDouble, _zoneType)) {
		_minY = tempDouble;
	}
	else _minY = SS_MIN_Y;

	if (Settings::settings->getSetting(getLocalType(), "MinZ", tempDouble, _zoneType)) {
		_minZ = tempDouble;
	}
	else _minZ = SS_MIN_Z;

	if (Settings::settings->getSetting(getLocalType(), "MaxX", tempDouble, _zoneType)) {
		_maxX = tempDouble;
	}
	else _maxX = SS_MAX_X;

	if (Settings::settings->getSetting(getLocalType(), "MaxY", tempDouble, _zoneType)) {
		_maxY = tempDouble;
	}
	else _maxY = SS_MAX_Y;

	if (Settings::settings->getSetting(getLocalType(), "MaxZ", tempDouble, _zoneType)) {
		_maxZ = tempDouble;
	}
	else _maxZ = SS_MAX_Z;

	if (Settings::settings->getSetting(getLocalType(), "WeightX", tempInt, _zoneType)) {
		_XWeight = tempInt;
	}
	else _XWeight = SS_XWEIGHT;
	
	if (Settings::settings->getSetting(getLocalType(), "WeightY", tempInt, _zoneType)) {
		_YWeight = tempInt;
	}
	else _YWeight = SS_YWEIGHT;
	
	if (Settings::settings->getSetting(getLocalType(), "WeightZ", tempInt, _zoneType)) {
		_ZWeight = tempInt;
	}
	else _ZWeight = SS_ZWEIGHT;
}