//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentType.h: interface for the AgentType class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AGENTTYPE_H__021A7751_5F71_11D4_9FA8_0000E87A7A07__INCLUDED_)
#define AFX_AGENTTYPE_H__021A7751_5F71_11D4_9FA8_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class Agent;
#include <vector>
#include <list>
#include "Settable.h"

using namespace std;

class AgentType: public Settable
{
public:
	AgentType();
	AgentType(Agent* seedAgent);
	virtual ~AgentType();

// Accessors

	string getName() {return _name;};
	bool getPerZoneType() {return _perZoneType;};
	int getWeight(int zoneType = 0);
	int getDeployed(int zoneType = 0);
	int getNZoneTypes() {return _nZoneTypes;};
	bool hasInterface() {return _hasInterface;};
	double getImpression(int zoneType, bool& valid);
	double getZoneTypeImpression(int zoneType, bool& valid);

	double	accumulate(double& totalWeight);	// Tally impressions of agents

// Mutators

	void setName(string name) {_name = name;};
	void setWeight(int weight, int zoneType = 0);
	void setDeployed(int deployed, int zoneType = 0);
	void setPerZoneType(bool perZoneType);	// Complicated, not implemented
	void addZoneType(int after);		// Complicated
	void removeZoneType(int which);		// Complicated
	void setNumZoneTypes(int newNumTypes, bool refreshSettings = false);
	void setImpression(double imp, int zoneType);
//Other
	void doInterface(int zoneType);
	void copyWeightAndDeploymentSettings(int copyZone);
	void copyAgentSettingsAcrossAllAgents(Agent* theAgent);
	void changeDeployment(int zoneType, int newDeployed);

// Settables inheritance
protected:

	virtual void reportSettings();
	virtual void receiveSettings();

private:
	void instantiateAgents(Agent* seedAgent);

	string			_name;
	bool			_perZoneType;
	vector<int>		_deployed;
	vector<int>		_weight;
	int				_nZoneTypes;
	bool			_hasInterface;
	int				_defaultDeployment;
	int				_defaultWeight;
	vector<double>	_impressions;

	vector<list<Agent*>*>		_myAgents;

};

#endif // !defined(AFX_AGENTTYPE_H__021A7751_5F71_11D4_9FA8_0000E87A7A07__INCLUDED_)
