//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Constants.h: contains all constants settable at the program scale.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CONSTANTS_H__INCLUDED_)
#define AFX_CONSTANTS_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// MATHEMATICAL CONSTANTS

const double PI = 3.14159265359;

// STARTING SHAPE CONSTANTS
const double STARTING_Z_HEIGHT = 14.0;
const double STARTING_Z_WIDENING = 1.0;

// Genetic Engine constants
const int MUTATION_RATE = 5;		// The maximum number of mutations to apply to any individual

const int POPULATION_SIZE = 20;  // The number of individuals in a population (can be overridden)

const int MINNUMBEROFSTARTINGTRANSFORMATIONS = 1; //the range of lengths of transformation lists of Designs used in seedInitialPopulation()
const int MAXNUMBEROFSTARTINGTRANSFORMATIONS = 5;
	
const int TOURNAMENTSIZE = 3; //number of individuals in each tournament used for fitness-proportionate selection

const int PARAMMUTATIONPERCENT = 20; //percent (out of 100) of individuals in a new population that will be modified by a mutation that changes a PrimaryShape's intrinsic parameters rather than its commands
  const int PETYPEPERCENT = 20; //percent of param mutations that affect PEType
  const int ZWIDENINGPERCENT = 20; //" " ZWidening
  const int SHAPEMODEPERCENT = 20; //" " shapeMode (1 - 45, determines how intersections are handled)
  const int ZHEIGHTPERCENT = 20; //" " ZHeight
  const int ZSTARTPERCENT = 20; //" " ZStart
	
const int INSERTIONMUTATIONPERCENT = 15; //percent (out of 100) of individuals in a new population that will be modified by a mutation that inserts a new command in the command list of a PrimaryShape
  const int MIDDLEPERCENT = 50; //percent of insertion mutations that insert a command in the middle of the PrimaryShape's command list
  const int ENDPERCENT = 50; //" " end of a PrimaryShape's command list
	
const int CHANGEMUTATIONPERCENT = 15; //percent (out of 100) of individuals in a new population that will be modified by a mutation that changes an existing command in a PrimaryShape's command list
  const int RINGPERCENT = 0; //percent of change mutations that alter a command's PERIng (does not apply to openings?)
  const int PARAMPERCENT = 50; //percent of change mutations that alter parameters of a command, such as the number of degrees of a rotation
  const int COMPLETEPERCENT = 50; //percent of change mutations that completely change a command

const int DELETEPERCENT = 15; // percent of individuals in a population that will be modified by a mutation that removes a command from one shape

const int CROSSOVERPERCENT = 20; //percent (out of 100) of individuals in a new population that will be created by crossing over the command lists of two individuals in the current population
  const int MINNUMBEROFCROSSINGPOINTS = 1; //minimum number of points at which the source of the new individual's transformation list is changed
  const int MAXNUMBEROFCROSSINGPOINTS = 10; //maximum number of points "" ""		
  const int MINCROSSOVERSIZE = 5; //DEFINE THIS BY WRITING CODE
  const int MAXCROSSOVERSIZE = 15; //DEFINE THIS BY WRITING CODE
	
const int REPRODUCTIONPERCENT = 15; //percent (out of 100) of individuals in a new population that will consist of unmodified members of the previous generation

const int ELITISM = 2; //number of individuals to be passed through unchanged
const int ELITECOPIES = 1; //number of copies of each elite individual to be passed into the next generation

// TRANSFORMATION CONSTANTS

// Mutation Constants
const int TRANSLATION_PERCENT = 29; //percent (out of 100) of mutations to operations that result in a translation
const int ROTATION_PERCENT = 28;
const int SCALE_PERCENT = 28;
const int REFLECTION_PERCENT = 0;
const int OPENING_PERCENT = 15;

// PEType constants
const int PETYPE_MIN = 0;  //JTR - I am here assuming that PETypes will effectively be represented by ints from 0 to PETYPE_MAX
const int PETYPE_MAX = 2; //this is arbitrary and should be changed when the PETypes are enumerated

//PERing constants
const int PERING_MIN = 1; //Minimum number of PETypes in a PERing
const int PERING_MAX = 10; //Maximum number of PETypes in a PERing

// ZWidening constants
const double Z_WIDENING_MIN = 0.8;
const double Z_WIDENING_MAX = 1.2;

// ZHeight constants
const double Z_HEIGHT_MIN = 0.0;
const double Z_HEIGHT_MAX = 20.0;

// ZStart constants
const double Z_START_MIN = 0.0;
const double Z_START_MAX = 20.0;

// Parameter scaling constants for rotation
const double ROTATION_PARAMETER_MIN = -15.0;
const double ROTATION_PARAMETER_MAX = 15.0;

// Parameter scaling constants for scaling
const double SCALING_PARAMETER_X_MIN = 0.8;
const double SCALING_PARAMETER_X_MAX = 1.2;
const double SCALING_PARAMETER_Y_MIN = 0.8;
const double SCALING_PARAMETER_Y_MAX = 1.2;
const double SCALING_PARAMETER_Z_MIN = 0.8;
const double SCALING_PARAMETER_Z_MAX = 1.2;

// Parameter scaling constants for translation
const double TRANSLATION_PARAMETER_X_MIN = -10.0;
const double TRANSLATION_PARAMETER_X_MAX = 10.0;
const double TRANSLATION_PARAMETER_Y_MIN = -10.0;
const double TRANSLATION_PARAMETER_Y_MAX = 10.0;
const double TRANSLATION_PARAMETER_Z_MIN = -10.0;
const double TRANSLATION_PARAMETER_Z_MAX = 10.0;

// Debugging
const int TRACE_LEVEL = 9; // 0 turns trace off, the higher the value, the more trace

#endif