//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// MEL-related utilities
//
//	Note that "Agency.mel" must first be sourced to get these to work.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "MELUtilities.h"
#include <sstream>
#include <iostream>

using namespace std;

void eval(string command) {

	//cout << "Running command: " << command << endl;

	MGlobal::executeCommand(command.c_str());
}


// Setting MEL values

void setMELvalue(string name, int value) {
	ostringstream command;
	command << "int " << name << " = " << value;
	eval(command.str());
}

void setMELvalue(string name, const vector<int> &value) {
	ostringstream command;

	command << "int " << name << "[] = {";
	for (int i = 0; i < value.size() - 1; i++)
		command << value[i] << ", ";
	command << value[value.size() - 1] << "}";
	eval(command.str());
}

void setMELvalue(string name, double value) {
	ostringstream command;
	command << "float " << name << " = " << value;
	eval(command.str());
}

void setMELvalue(string name, const vector<double> &value) {
	ostringstream command;

	command << "float " << name << "[] = {";
	for (int i = 0; i < value.size() - 1; i++)
		command << value[i] << ", ";
	command << value[value.size() - 1] << "}";
	eval(command.str());
}

void setMELvalue(string name, string value) {
	string command = "string " + name + " = \"" + value +"\"";
	eval(command);
}

void setMELvalue(string name, const vector<string> &value) {
	ostringstream command;

	command << "string " << name << "[] = {";
	for (int i = 0; i < value.size() - 1; i++)
		command << "\"" << value[i] << "\", ";
	command << "\"" << value[value.size() - 1] << "\"}";
	eval(command.str());
}


// Utilities to get MEL values from internal variables

void getMELvalue(string name, int &value) {
	string command = "return_int " + name;

	MGlobal::executeCommand(command.c_str(), value);
}
	
void getMELvalue(string name, vector<int> &value) {
	MIntArray	result;
	string		command = "return_int_array " + name;

	MGlobal::executeCommand(command.c_str(), result);

	value.resize(result.length());
	for (int i = 0; i < result.length(); i++)
		value[i] = result[i];
}

void getMELvalue(string name, double &value) {
	string command = "return_float " + name;

	MGlobal::executeCommand(command.c_str(), value);
}

void getMELvalue(string name, vector<double> &value) {
	MDoubleArray	result;
	string		command = "return_float_array " + name;

	MGlobal::executeCommand(command.c_str(), result);

	value.resize(result.length());
	for (int i = 0; i < result.length(); i++)
		value[i] = result[i];
}

void getMELvalue(string name, string &value) {
	string command = "return_string " + name;
	MString result;

	MGlobal::executeCommand(command.c_str(), result);
	value = result.asChar();
}

void getMELvalue(string name, vector<string> &value) {
	MStringArray	result;
	string		command = "return_string_array " + name;

	MGlobal::executeCommand(command.c_str(), result);

	value.resize(result.length());
	for (int i = 0; i < result.length(); i++)
		value[i] = (result[i]).asChar();
}


// Utilities to return MEL values directly
int getMELint(string name) {
	int value;
	getMELvalue(name, value);
	return value;
}

vector<int>& getMELintVector(string name) {
	vector<int>* value = new vector<int>;
	getMELvalue(name, *value);
	return *value;
}

double			getMELdouble(string name) {
	double value;
	getMELvalue(name, value);
	return value;
}

vector<double>&	getMELdoubleVector(string name) {
	vector<double>* value = new vector<double>;
	getMELvalue(name, *value);
	return *value; 
}

string getMELstring(string name) {
	string value;
	getMELvalue(name, value);
	return value;
}

vector<string>& getMELstringVector(string name) {
	vector<string>* value = new vector<string>;
	getMELvalue(name, *value);
	return *value; 
}