// MainControls.cpp : implementation file
//

#include "stdafx.h"
#include "MFCPlugin.h"
#include "MainControls.h"
#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MDagPath.h>
#include <maya/MFnDagNode.h>
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
#include <maya/MFnTransform.h>
#include "Utilities.h"
#include "MELUtilities.h"
#include <iostream>
#include <sstream>
#include <time.h>
#include "Random.h"
#include "Constants.h"
#include "GeneticEngine.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MainControls dialog


MainControls::MainControls(CWnd* pParent /*=NULL*/)
	: CDialog(MainControls::IDD, pParent)
{
	//{{AFX_DATA_INIT(MainControls)
	m_nGenerations = 0;
	//}}AFX_DATA_INIT

    MSelectionList      list;
	MStringArray		names;
	std::vector<string> shapeNames;
	

	currentDesign = 0;

// This code gets the Maya curves selected

	int selCount = 0;
    MGlobal::getActiveSelectionList( list );
	for ( MItSelectionList listIter( list, MFn::kNurbsCurve ); !listIter.isDone(); listIter.next() )
	{
		selCount++;
	}

	if (selCount == 0) {
		cout << "No NURBS curves selected." << endl;
		theEngine = new GeneticEngine(1);
		return;
	}

	shapeNames.resize(selCount);
	int i = 0;
	for ( listIter.reset(); !listIter.isDone(); listIter.next() )
	{
		listIter.getStrings(names);
		shapeNames[i] = (names[0]).asChar();
		i++;
	}

	theEngine = new GeneticEngine(POPULATION_SIZE, shapeNames);

}

MainControls::~MainControls() {
	delete theEngine;
}

void MainControls::updateDialogText() {
	ostringstream genStr, popStr, meanStr, devStr, indStr, fitStr;

	genStr << "Generation: " << theEngine->getGenerationsRun();
	SetDlgItemText(IDC_GENERATION, genStr.str().c_str());

	popStr << "Population Size: " << theEngine->getPopulationSize();
	SetDlgItemText(IDC_POP_SIZE, popStr.str().c_str());

	meanStr << "Mean Unfitness: " << theEngine->getMeanFitness();
	SetDlgItemText(IDC_MEAN_FITNESS, meanStr.str().c_str());

	devStr << "Standard Deviation: " << theEngine->getStdDevFitness();
	SetDlgItemText(IDC_STD_DEV, devStr.str().c_str());

	indStr << "Individual: " << currentDesign + 1;
	SetDlgItemText(IDC_INDIVIDUAL, indStr.str().c_str());
	
	fitStr << "Unfitness: " << theEngine->getPopulation()->getDesign(currentDesign)->getRawFitness();
	SetDlgItemText(IDC_FITNESS, fitStr.str().c_str());
}

void MainControls::updateBreedingText() {
	CEdit* editBox = (CEdit*)GetDlgItem(IDC_EDIT1);
	CString oldText;
	string newText;

	editBox->GetWindowText(oldText);
	theEngine->returnStats(newText);
	editBox->SetWindowText(oldText + newText.c_str());
	editBox->LineScroll(editBox->GetLineCount() - 1);
}

void MainControls::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MainControls)
	DDX_Text(pDX, IDC_N_GENERATIONS_EDIT, m_nGenerations);
	DDV_MinMaxInt(pDX, m_nGenerations, 1, 10000);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MainControls, CDialog)
	//{{AFX_MSG_MAP(MainControls)
	ON_BN_CLICKED(IDC_TRANSFORM, OnTransform)
	ON_BN_CLICKED(IDC_STARTGP, OnStart)
	ON_CBN_SELENDOK(IDC_SELECT_INDIVIDUAL, OnSelendokSelectIndividual)
	ON_CBN_DROPDOWN(IDC_SELECT_INDIVIDUAL, OnDropdownSelectIndividual)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SHIFT_INDIVIDUAL, OnDeltaposShiftIndividual)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MainControls message handlers

void MainControls::OnTransform() 
{
/*	double x, y, z;
	double test;
	ostringstream output;
	CEdit* editBox = (CEdit*)GetDlgItem(IDC_EDIT1);
	CString oldText;

	editBox->GetWindowText(oldText);

	for (int i = 0; i < primaries.size(); i++) {

	test = Random::genrand() * 4;

	if (test > 3.0) {
		x = (Random::genrand() * (Z_WIDENING_MAX - Z_WIDENING_MIN)) + Z_WIDENING_MIN;
		primaries[i]->setZWidening(x);

		output << "Shape \"" << primaries[i]->getMayaName() << "\" Set Z Widening (" << x << ").\r\n";
	}
	if (test > 2.0) {
		x = (Random::genrand() * (TRANSLATION_PARAMETER_X_MAX - TRANSLATION_PARAMETER_X_MIN)) + TRANSLATION_PARAMETER_X_MIN;
		y = (Random::genrand() * (TRANSLATION_PARAMETER_Y_MAX - TRANSLATION_PARAMETER_Y_MIN)) + TRANSLATION_PARAMETER_Y_MIN;
		z = (Random::genrand() * (TRANSLATION_PARAMETER_Z_MAX - TRANSLATION_PARAMETER_Z_MIN)) + TRANSLATION_PARAMETER_Z_MIN;
		primaries[i]->translate(x, y, z);

		output << "Shape \"" << primaries[i]->getMayaName() << "\" translated (" << x << ", " << y << ", " << z << ").\r\n";
	}
	else if (test > 1.0) {
		x = (Random::genrand() * (SCALING_PARAMETER_X_MAX - SCALING_PARAMETER_X_MIN)) + SCALING_PARAMETER_X_MIN;
		y = (Random::genrand() * (SCALING_PARAMETER_Y_MAX - SCALING_PARAMETER_Y_MIN)) + SCALING_PARAMETER_Y_MIN;
		z = (Random::genrand() * (SCALING_PARAMETER_Z_MAX - SCALING_PARAMETER_Z_MIN)) + SCALING_PARAMETER_Z_MIN;
		primaries[i]->scale(x, y, z);

		output << "Shape \"" << primaries[i]->getMayaName() << "\" scaled (" << x << ", " << y << ", " << z << ").\r\n";
	}
	else
		x = Random::genrand() * ROTATION_PARAMETER_MAX * 2 - ROTATION_PARAMETER_MAX;
		primaries[i]->rotate(x);

		output << "Shape \"" << primaries[i]->getMayaName() << "\" rotated by (" << x << ").\r\n";
	}

	editBox->SetWindowText(oldText + output.str().c_str() + "\r\n");
	editBox->LineScroll(editBox->GetLineCount());

	
	CButton* checkBox = (CButton*)GetDlgItem(IDC_START_BOTTOM);
	
	deleteLoft();
	for (i = 0; i < primaries.size(); i++)
		primaries[i]->loft(checkBox->GetCheck(), loftShapeList);
	
	eval("refresh");*/
}

void MainControls::OnOK() 
{
	
	CDialog::OnOK();
}

void MainControls::interpret() {
	CButton* checkBox = (CButton*)GetDlgItem(IDC_START_BOTTOM);

	deleteLoft();
	theEngine->getPopulation()->getDesign(currentDesign)->interpret(checkBox->GetCheck(), loftShapeList);
	eval("refresh");
}

void MainControls::deleteLoft() {

	MStatus				status;

	status = MGlobal::clearSelectionList();
	for (list<string>::const_iterator iter = loftShapeList.begin(); iter != loftShapeList.end(); iter++)
		status = MGlobal::selectByName((*iter).c_str());
	eval("delete;");

	loftShapeList.clear();


}

void MainControls::OnCancel() 
{
	deleteLoft();	
	CDialog::OnCancel();
}

void MainControls::OnStart()
{
	theEngine->runPopulation(1);
	updateDialogText();
	updateBreedingText();
	interpret();
}

BOOL MainControls::OnInitDialog() 
{
	CDialog::OnInitDialog();
	char buffer[20];
	
	CComboBox* indList = (CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL);
	indList->ResetContent();
	for (int i = 0; i < theEngine->getPopulationSize(); i++) {
		itoa(i+1, buffer, 10);
		indList->AddString(buffer);
	}
	indList->SetCurSel(0);

	updateDialogText();

	interpret();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void MainControls::OnSelendokSelectIndividual() 
{
	currentDesign = ((CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL))->GetCurSel();
	updateDialogText();
	interpret();
}

void MainControls::OnDropdownSelectIndividual() 
{
	// TODO: Add your control notification handler code here
	
}

void MainControls::OnDeltaposShiftIndividual(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;

	CSpinButtonCtrl* spin = (CSpinButtonCtrl*)GetDlgItem(IDC_SHIFT_INDIVIDUAL);
	CComboBox* indList = (CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL);

	int curPos = indList->GetCurSel();

	if ((curPos + pNMUpDown->iDelta) < 0) return;
	if ((curPos + pNMUpDown->iDelta) > (theEngine->getPopulationSize() - 1)) return;

	indList->SetCurSel(curPos + pNMUpDown->iDelta);
	OnSelendokSelectIndividual();

	*pResult = 0;
}

void MainControls::OnAbort() 
{
	// TODO: Add your control notification handler code here
	
}
