//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Maya-related utilities
//
//	Note that "Agency.mel" must first be sourced to get these to work.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MayaUtilities.h"
#include "MELUtilities.h"
#include "MayaInfo.h"
#include "ShapePair.h"
#include "Constants.h"
#include "ZoneTypes.h"
#include <sstream>
#include <iostream>

bool intersect(string curve1, string curve2)	// Relies on the MEL procedure "intersects" defined in Agency.mel
{
	eval("int $itest = intersects(\"" + curve1 + "\", \"" + curve2 + "\")");
	return getMELint("$itest");
}

bool planarIntersect(Shape* curve1, Shape* curve2)
{
	return intersect(curve1->getName(), curve2->getName());
}

void getIntersections(string curve1, string curve2, vector<double>& p1, vector<double>& p2)
{	eval("string $node = `curveIntersect -ch on \"" + curve1 + "\" \"" + curve2 +"\"`;");
	eval("float $p1[] = `getAttr ($node + \".parameter1\")`;");
	eval("float $p2[] = `getAttr ($node + \".parameter2\")`;");
	eval("eval(\"delete \" + $node);");

	getMELvalue("$p1", p1);
	getMELvalue("$p2", p2);
}

void sliceCurve(string curve, vector<double>& cuts, vector<string>& newCurves)	// This slices a curve into a ((cuts.size()-1)*2) new curves based on cutpoints
{
	ostringstream command;
	
//	newCurves.resize((cuts.size()-1)*2);
	vector<string> dcurves;
	string whatType;

	for (int j = 0; j < cuts.size() - 1; j++) {
		command.str("");

		command << "string $dcurves[] = `detachCurve -ch on";

		for (int i = j + 1; i >= j; i--)
			command << " -p "<< cuts[i];

		command << " -rpo off " << "\"" << curve << "\"`";
		eval(command.str());
		getMELvalue("$dcurves", dcurves);
		for (int k = dcurves.size() - 1; k >= 0 ; k--) {
			command.str("");
			command << "string $whatType = `nodeType \"" + dcurves[k] + "\"`";
			eval(command.str());
			getMELvalue("$whatType", whatType);
			if (whatType == "transform") {
				newCurves.push_back(dcurves[k]);
				eval("delete -ch " + dcurves[k]);
			}
			else {
				eval("delete " + dcurves[k]);
			}
		}
	}
}

void cutCurve(string curve, double start, double stop)	// This slices a curve into a ((cuts.size()-1)*2) new curves based on cutpoints
{

	ostringstream command;

	command << "detachCurve -ch off -p " << start << " -p " << stop << " -rpo on " << curve;
	eval(command.str());
}

void sewCurves(int method, const string& A, const string& B, const string& C, const string& D, vector<string>& newCurves)	// method is 1 - 45, of possible combinatorics
{
	vector<string> newBCurves;
	sewCurves(method, A, B, C, D, newCurves, newBCurves);
	for (int i = 0; i < newBCurves.size(); i++)
		newCurves.push_back(newBCurves[i]);
}

void sewCurves(int method, const string& A, const string& B, const string& C, const string& D, vector<string>& newACurves, vector<string>& newBCurves)	// method is 1 - 45, of possible combinatorics
{
	string c1, c2, c3, c4;

	switch ((method % 15) + 1) {
	case 1:
		c1 = A;
		c2 = B;
		c3 = A;
		c4 = C;
		break;
	case 2:
		c1 = A;
		c2 = B;
		c3 = A;
		c4 = D;
		break;
	case 3:
		c1 = A;
		c2 = B;
		c3 = B;
		c4 = C;
		break;
	case 4:
		c1 = A;
		c2 = B;
		c3 = B;
		c4 = D;
		break;
	case 5:
		c1 = A;
		c2 = B;
		c3 = C;
		c4 = D;
		break;
	case 6:
		c1 = A;
		c2 = C;
		c3 = A;
		c4 = D;
		break;
	case 7:
		c1 = A;
		c2 = C;
		c3 = B;
		c4 = C;
		break;
	case 8:
		c1 = A;
		c2 = C;
		c3 = B;
		c4 = D;
		break;
	case 9:
		c1 = A;
		c2 = C;
		c3 = C;
		c4 = D;
		break;
	case 10:
		c1 = A;
		c2 = D;
		c3 = B;
		c4 = C;
		break;
	case 11:
		c1 = A;
		c2 = D;
		c3 = B;
		c4 = D;
		break;
	case 12:
		c1 = A;
		c2 = D;
		c3 = C;
		c4 = D;
		break;
	case 13:
		c1 = B;
		c2 = C;
		c3 = B;
		c4 = D;
		break;
	case 14:
		c1 = B;
		c2 = C;
		c3 = C;
		c4 = D;
		break;
	case 15:
		c1 = B;
		c2 = D;
		c3 = C;
		c4 = D;
		break;
	}

	string command, acurve;

	if (method < 16 || method > 30) {
		command = "string $acurves[] = `attachCurve -ch 0 -rpo 0 -kmk 0 -m 0 -bb 0.5 -bki 0 -p 0.1 \"" + c1 + "\" \"" + c2 + "\"`";
		eval(command);
		getMELvalue("$acurves[0]", acurve);
		newACurves.push_back(acurve);
	}

	if (method < 31) {
		command = "string $bcurves[] = `attachCurve -ch 0 -rpo 0 -kmk 0 -m 0 -bb 0.5 -bki 0 -p 0.1 \"" + c3 + "\" \"" + c4 + "\"`";
		eval(command);
		getMELvalue("$bcurves[0]", acurve);
		newBCurves.push_back(acurve);	
	}
}

void	shuffleIntersections(int method, const string& curve1, const string& curve2, vector<string>& newACurves, vector<string>& newBCurves)
{	
	if (intersect(curve1, curve2)) {

		vector<double> p1;
		vector<double> p2;
		getIntersections(curve1, curve2, p1, p2);


		vector<string> newCurves1, newCurves2;
		sliceCurve(curve1, p1, newCurves1);
		sliceCurve(curve2, p2, newCurves2);

		newACurves.resize(0);
		newBCurves.resize(0);
		for (int i = 0; i < newCurves1.size() / 2; i++) {
			sewCurves(method, newCurves1[i*2], newCurves1[i * 2 + 1], newCurves2[i*2], newCurves2[i * 2 + 1], newACurves, newBCurves);
		}

		for (i = 0; i < newCurves1.size(); i++) {
			eval("delete " + newCurves1[i]);
			eval("delete " + newCurves2[i]);
		}
	}
}

void	shuffleIntersections(int method, Shape* curve1, Shape* curve2, vector<string>& newACurves, vector<string>& newBCurves)
{
	shuffleIntersections(method, curve1->getName(), curve2->getName(), newACurves, newBCurves);
}

void shuffleIntersections(int method, Shape* curve1, Shape* curve2, list<EmergentShape*>& newCurves)
{
	vector<string> newACurves, newBCurves;
	EmergentShape* newShape;
	newCurves.clear();

	shuffleIntersections(method, curve1, curve2, newACurves, newBCurves);
	for (int i = 0; i < newACurves.size(); i++) {
		newShape = new EmergentShape(*curve1);
		newShape->setMayaName(newACurves[i]);
		newCurves.push_back(newShape);
	}
	for (i = 0; i < newBCurves.size(); i++) {
		newShape = new EmergentShape(*curve2);
		newShape->setMayaName(newBCurves[i]);
		newCurves.push_back(newShape);
	}
}

bool ZSection(Shape* A, Shape* B, Shape*& newA, Shape*& newB, ShapePair& intersectors, bool startFromZero)



{
	double ALow, BLow;

	if (startFromZero) {
		ALow = 0.0;
		BLow = 0.0;
	}
	else {
		ALow = A->getZStart();
		BLow = B->getZStart();
	}

	double AHigh = ALow + A->getZHeight();
	double BHigh = BLow + B->getZHeight();
	newA = NULL;
	newB = NULL;

	if ( (BHigh <= ALow) || (AHigh <= BLow) ) {		// A and B don't intersect in Z
		intersectors.set(NULL, NULL);
		return false;
	}

	if (ALow == BLow) {
		if (AHigh == BHigh) {			// Same start and height
			intersectors.set(A, B);
			return false;
		}
		if (AHigh > BHigh) {
			intersectors.set(A->ZCut(BHigh, UPPER, startFromZero), B);	// bases same, A taller
			return true;
		}
		if (BHigh > AHigh) {
			intersectors.set(A, B->ZCut(AHigh, UPPER, startFromZero));	// bases same, B taller
			return true;
		}
	}
	if (ALow > BLow) {			// A starts higher than B
		if (AHigh == BHigh) {
			intersectors.set(A, B->ZCut(ALow, LOWER, startFromZero));	// tops same, A starts higher
			return true;
		}
		if (AHigh > BHigh) {
			intersectors.set(A->ZCut(BHigh, UPPER, startFromZero), B->ZCut(ALow, LOWER, startFromZero)); // A starts higher, A ends higher
			return true;
		}
		if (BHigh > AHigh) {								// A starts higher and ends lower
			newB = B->ZCut(AHigh, LOWER, startFromZero);				// this cuts B into 3 pieces
			intersectors.set(A, B->ZCut(ALow, LOWER, startFromZero));
			return true;
		}
	}
	if (ALow < BLow) {			// B starts higher
		if (AHigh == BHigh) {
			intersectors.set(A->ZCut(BLow, LOWER, startFromZero), B);	// B starts higher, tops the same
			return true;
		}
		if (AHigh > BHigh) {		// B starts higher and ends lower
			newA = A->ZCut(BHigh, LOWER, startFromZero);		// this cuts A into three pieces
			intersectors.set(A->ZCut(BLow, LOWER, startFromZero), B);
			return true;
		}
		if (AHigh < BHigh) {		// B starts higher and ends higher
			intersectors.set(A->ZCut(BLow, LOWER, startFromZero), B->ZCut(AHigh, UPPER, startFromZero));
			return true;
		}
	}

	return false;
	// That's it.
}

bool duplicateCurve(MayaInfo* curve, string& newName) {  // Makes a copy of the original Maya object to operate on safely
	MStatus				status;
	MDagPath			dup1;
	MObject				dupObject1, dupTransform1;
	int					i;

	if (!curve->node()) {
		cout << "No shape named " << curve->name() << endl;
		return false;
	}

	MFnDagNode		fnDagNode(*curve->node(), &status);
	if (!status) {
		status.perror("MFnDagNode constructor");
		return false;
	}

	dupObject1 = fnDagNode.duplicate ( FALSE, FALSE, &status );  // I have no idea what these parameters mean
	if (!status) {
		status.perror("MFnDagNode duplicate");
		return false;
	}

	status = MDagPath::getAPathTo ( dupObject1, dup1 );
	if (!status) {
		status.perror("MDagPath getAPathTo");
		return false;
	}

	newName = dup1.partialPathName(&status).asChar();
	return true;
}

bool layerExists(const string& name)
{
	int noLayer;
	eval("int $noLayer = catch(`getAttr " + name + ".id`)");
	getMELvalue("$noLayer", noLayer);
	return (noLayer == false);
}

void newLayer(string& name)				// NOTE: Name will be changed if layer by "name" already exists
{
	ostringstream newName, command;
	int i = 1;


	newName << name;
	while (layerExists(newName.str())) {
		i++;
		newName.str("");
		newName << name << "Copy" << i;
	}
	name = newName.str();

	eval("createDisplayLayer -empty -n " + name);
}

void setLayerVisibility(const string& name, bool vis)
{
	if (vis) eval("setAttr " + name + ".visibility 1");
	else eval("setAttr " + name + ".visibility 0");
}

void getCurrentLayer(string& name)
{
	eval("string $curLayer = `editDisplayLayerGlobals -q -cdl`");
	getMELvalue("$curLayer", name);
}

void setCurrentLayer(const string& name)
{
	eval("editDisplayLayerGlobals -cdl " + name);
}

void selectAllOnLayer(const string& name) {
	eval("select `editDisplayLayerMembers -q " + name + "`");
}

void deleteAllOnLayer(const string& name) {
	selectAllOnLayer(name);
	eval("delete");
}

void initializeShaderGroups()
{
	ostringstream command;
	int noShader;
	vector<string> shaders;

	eval("string $shaders[] = `ls -mat`");
	getMELvalue("$shaders", shaders);

	for (int i = 0; i < shaders.size(); i++) {

		command.str("");
		command << "int $noShader = catch(`select " << shaders[i] << "SG`)";

		eval(command.str());
		getMELvalue("$noShader", noShader);
		if (noShader) {
			command.str("");
			command << "sets -renderable true -noSurfaceShader true -empty -name " << shaders[i] << "SG";
			eval(command.str());
			command.str("");
			command << "connectAttr -f " << shaders[i] << ".outColor " << shaders[i] << "SG.surfaceShader";
			eval(command.str());
		}
		command.str("");
		command << shaders[i] << "SG";
		ZoneTypes::zoneTypes->addShader(command.str());
	}
}