//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Message.h: interface for the Message class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MESSAGE_H__35068A30_3E41_11D4_9F88_0000E87A7A07__INCLUDED_)
#define AFX_MESSAGE_H__35068A30_3E41_11D4_9F88_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Agent.h"
#include "Filter.h"
#include <string>
#include "MessageContentTypes.h"

using namespace std;

class Message  // Message should be an abstract class. Each type of message should be its own class.
{				// The content of the message must be settable and retrievable by appropriate type per class
public:
	Message();

	Message(string messageString);
	Message(int	messageInt);
	Message(double messageDouble);
	Message(stringInt* messageStringInt);
	

	virtual ~Message();

	bool isTo(AgentID theAgent); // Tests whether this message goes to a certain agentID
	bool isTo(string theType); // Tests whether this message goes to a certain agent type

	void clear();	// Clears content, contentType, and filter

	string&	getContentType() {return contentType;};
	void*	getContent() {return content;};

	void setContent(void* newContent) {content = newContent;};
	void setContentType(const string& newContentType) {contentType = newContentType;};

	void addRecipient(AgentID recipient);
	void addRecipientType(string recipientType);

private:

	Agent*	from;
	Filter	to;
	string	contentType;	// Tell the recipient the type that the content points to
	void*	content;

};

#endif // !defined(AFX_MESSAGE_H__35068A30_3E41_11D4_9F88_0000E87A7A07__INCLUDED_)
