//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Miscellaneous.cpp : implementation file
//

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "mfcplugin.h"
#include "Miscellaneous.h"
#include "MELUtilities.h"
#include "MayaUtilities.h"
#include <sstream>
#include "Math.h"
#include <string>
#include <vector>
#include "PopulationProperties.h"
#include "Shape.h"
#include "Phenotype.h"
#include "GeneticEngine.h"
#include <iostream>

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous property page

IMPLEMENT_DYNCREATE(Miscellaneous, CPropertyPage)

Miscellaneous::Miscellaneous() : CPropertyPage(Miscellaneous::IDD)
{
	//{{AFX_DATA_INIT(Miscellaneous)
	m_size = 0;
	m_zoom = 30;
	m_azimuth = 50;
	m_elevation = 50;
	//}}AFX_DATA_INIT
	_xpos = 0;
	_ypos = 0;
	_zpos = 0;
	_xsize = 10;
	_ysize = 10;
	_zsize = 10;
	_dir = 0;

	_zoom = 30;
	_azimuth = 50;
	_elevation = 50;
}

Miscellaneous::~Miscellaneous()
{
	deletePlane();
	int originala, originale, originalz = 30 - _zoom;
	ostringstream originalzoom, originalangle;
	originalzoom << "dolly -rel -d " << originalz << " persp";
	eval(originalzoom.str());
	
	originala = fabs(3.6*(50 - _azimuth));
	originale = fabs(3.6*(50 - _elevation));
	if(_azimuth >= 0 && _elevation >= 0)
		originalangle << "tumble -ra -" << originala << " -" << originale;
	else if(_azimuth >= 0 && _elevation < 0)
		originalangle << "tumble -ra -" << originala << " " << originale;
	else if(_azimuth < 0 && _elevation >= 0)
		originalangle << "tumble -ra " << originala << " -" << originale;
	else
		originalangle << "tumble -ra " << originala << " " << originale;

	eval(originalangle.str());
	eval("refresh");
}

void Miscellaneous::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(Miscellaneous)
	DDX_Slider(pDX, IDC_PLANE_SIDE, m_size);
	DDX_Slider(pDX, IDC_ZOOM, m_zoom);
	DDX_Slider(pDX, IDC_AZIMUTH, m_azimuth);
	DDX_Slider(pDX, IDC_ELEVATION, m_elevation);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(Miscellaneous, CPropertyPage)
	//{{AFX_MSG_MAP(Miscellaneous)
	ON_NOTIFY(UDN_DELTAPOS, IDC_POSITION, OnDeltaposPosition)
	ON_BN_CLICKED(IDC_X, OnX)
	ON_BN_CLICKED(IDC_Y, OnY)
	ON_BN_CLICKED(IDC_Z, OnZ)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_PLANE_SIDE, OnCustomdrawPlaneSide)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_ZOOM, OnCustomdrawZoom)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_AZIMUTH, OnCustomdrawAzimuth)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_ELEVATION, OnCustomdrawElevation)
	ON_BN_CLICKED(IDC_RESTORE, OnRestore)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous message handlers




void Miscellaneous::OnDeltaposPosition(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	if(_dir == 0)
		_xpos -= pNMUpDown->iDelta;
	else if(_dir == 1)
		_ypos -= pNMUpDown->iDelta;
	else
		_zpos -= pNMUpDown->iDelta;

	deletePlane();
	drawPlane();
	
	*pResult = 0;
}

BOOL Miscellaneous::OnInitDialog() 
{
	_xpos = 0;
	_ypos = 0;
	_zpos = 0;
	_xsize = 0;
	_ysize = 0;
	_zsize = 0;
	m_size = 0;
	_dir = 2;

	_zoom = 30;
	m_zoom = 30;
	_azimuth = 50;
	m_azimuth = 50;
	_elevation = 50;
	m_elevation = 50;

	((CButton*)GetDlgItem(IDC_X))->SetCheck(0);
	((CButton*)GetDlgItem(IDC_Y))->SetCheck(0);
	((CButton*)GetDlgItem(IDC_Z))->SetCheck(1);

	CPropertyPage::OnInitDialog();
	
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void Miscellaneous::drawPlane()
{
	ostringstream side, pos;
	string move = "setAttr plane.translate";
	if(_dir == 0)	{
		side << _xsize;
		pos << fabs(_xpos);
		eval("nurbsPlane -ch off -n plane -ax 0 0 1 -po 2 -w " + side.str());
		if(_xpos < 0)
			move += "Z -" + pos.str();
		else
			move += "Z " + pos.str();
		eval(move);
	}
	else if(_dir == 1)	{
		side << _ysize;
		pos << fabs(_ypos);
		eval("nurbsPlane -ch off -n plane -ax 1 0 0 -po 2 -w " + side.str());
		if(_ypos < 0)
			move += "X -" + pos.str();
		else
			move += "X " + pos.str();
		eval(move);
	}
	else	{
		side << _zsize;
		pos << fabs(_zpos);
		eval("nurbsPlane -ch off -n plane -ax 0 1 0 -po 2 -w " + side.str());
		if(_zpos < 0)
			move += "Y -" + pos.str();
		else
			move += "Y " + pos.str();
		eval(move);
	}
	
	eval("refresh");
}

BOOL Miscellaneous::OnKillActive() 
{
	deletePlane();
	

	return CPropertyPage::OnKillActive();
}

void Miscellaneous::deletePlane()
{
	eval("delete plane");
	eval("refresh");
}

BOOL Miscellaneous::OnSetActive() 
{
	_xpos = 0;
	_ypos = 0;
	_zpos = 0;
	_xsize = 0;
	_ysize = 0;
	_zsize = 0;
	m_size = 0;
	_dir = 2;

	

	((CButton*)GetDlgItem(IDC_X))->SetCheck(0);
	((CButton*)GetDlgItem(IDC_Y))->SetCheck(0);
	((CButton*)GetDlgItem(IDC_Z))->SetCheck(1);
	return CPropertyPage::OnSetActive();
}

void Miscellaneous::OnX() 
{
	_dir = 0;
	m_size = _xsize;
	deletePlane();
	drawPlane();
	UpdateData(false);
	
}

void Miscellaneous::OnY() 
{
	_dir = 1;
	m_size = _ysize;
	deletePlane();
	drawPlane();
	UpdateData(false);
	
}

void Miscellaneous::OnZ() 
{
	_dir = 2;
	m_size = _zsize;
	deletePlane();
	drawPlane();
	UpdateData(false);
	
}

void Miscellaneous::OnCustomdrawPlaneSide(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	if(_dir == 0)
		_xsize = m_size;
	else if(_dir == 1)
		_ysize = m_size;
	else
		_zsize = m_size;

	deletePlane();
	drawPlane();
	
	*pResult = 0;
}


void Miscellaneous::OnCustomdrawZoom(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	
	ostringstream	zoom;
	int dolly = fabs(m_zoom - _zoom);
	if(m_zoom - _zoom >= 0)
		zoom << "dolly -rel -d " << dolly << " persp";
	else
		zoom << "dolly -rel -d -" << dolly << " persp";

	eval(zoom.str());
	_zoom = m_zoom;
	eval("refresh");
	
	*pResult = 0;
}

void Miscellaneous::OnCustomdrawAzimuth(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();

	ostringstream rotate;
	int az = 3.6*fabs(m_azimuth - _azimuth);
	if(m_azimuth - _azimuth >= 0)
		rotate << "tumble -aa " << az << " persp";
	else
		rotate << "tumble -aa -" << az << " persp";

	eval(rotate.str());
	_azimuth = m_azimuth;
	eval("refresh");

	*pResult = 0;
}

void Miscellaneous::OnCustomdrawElevation(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();

	ostringstream rotate;
	int el = 3.6*fabs(m_elevation - _elevation);
	if(m_elevation - _elevation >= 0)
		rotate << "tumble -ea " << el << " persp";
	else
		rotate << "tumble -ea -" << el << " persp";

	eval(rotate.str());
	_elevation = m_elevation;
	eval("refresh");

	*pResult = 0;
}

void Miscellaneous::OnRestore() 
{
	int originala, originale, originalz = 30 - _zoom;
	ostringstream originalzoom, originalangle;
	originalzoom << "dolly -rel -d " << originalz << " persp";
	eval(originalzoom.str());
	
	originala = 3.6*fabs(50 - _azimuth);
	originale = 3.6*fabs(50 - _elevation);
	if(50 - _azimuth >= 0 && 50 - _elevation >= 0)
		originalangle << "tumble -ra " << originala << " " << originale;
	else if(50 - _azimuth >= 0 && 50 - _elevation < 0)
		originalangle << "tumble -ra " << originala << " -" << originale;
	else if(50 - _azimuth < 0 && 50 - _elevation >= 0)
		originalangle << "tumble -ra -" << originala << " " << originale;
	else
		originalangle << "tumble -ra -" << originala << " -" << originale;

	eval(originalangle.str());
	eval("refresh");

	_zoom = 30;
	m_zoom = 30;
	_azimuth = 50;
	m_azimuth = 50;
	_elevation = 50;
	m_elevation = 50;
	UpdateData(false);

}
