//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// ModifyIndividualDialog.cpp : implementation file
//

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "ModifyIndividualDialog.h"
#include "PopulationProperties.h"
#include "Design.h"
#include "Genotype.h"
#include "PrimaryShape.h"
#include <iostream>
#include <sstream>
#include "Constants.h"
#include "MELUtilities.h"
#include "ZoneTypes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// ModifyIndividualDialog dialog


ModifyIndividualDialog::ModifyIndividualDialog(CWnd* pParent /*=NULL*/)
	: CDialog(ModifyIndividualDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(ModifyIndividualDialog)
	m_edit1 = 0.0;
	m_edit2 = 0.0;
	m_edit3 = 0.0;
	m_newFitness = 0.0;
	m_copies = 0;
	//}}AFX_DATA_INIT
}


void ModifyIndividualDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(ModifyIndividualDialog)
	DDX_Text(pDX, IDC_EDIT1, m_edit1);
	DDX_Text(pDX, IDC_EDIT2, m_edit2);
	DDX_Text(pDX, IDC_EDIT3, m_edit3);
	DDX_Text(pDX, IDC_MODIFY_FITNESS, m_newFitness);
	DDX_Text(pDX, IDC_MAKE_COPIES, m_copies);
	//}}AFX_DATA_MAP

	DDV_MinMaxDouble(pDX, m_edit1, edit1min, edit1max);
	DDV_MinMaxDouble(pDX, m_edit2, edit2min, edit2max);
	DDV_MinMaxDouble(pDX, m_edit3, edit3min, edit3max);
	DDV_MinMaxInt(pDX, m_copies, 1, GeneticEngine::geneticEngine->getPopulationSize());
}


BEGIN_MESSAGE_MAP(ModifyIndividualDialog, CDialog)
	//{{AFX_MSG_MAP(ModifyIndividualDialog)
	ON_BN_CLICKED(IDDONE, OnDone)
	ON_NOTIFY(TVN_SELCHANGED, IDC_INDIVIDUAL_TREE, OnSelchangedIndividualTree)
	ON_WM_DESTROY()
	ON_EN_UPDATE(IDC_EDIT1, OnUpdateEdit1)
	ON_EN_UPDATE(IDC_EDIT2, OnUpdateEdit2)
	ON_EN_UPDATE(IDC_EDIT3, OnUpdateEdit3)
	ON_NOTIFY(TVN_SELCHANGING, IDC_INDIVIDUAL_TREE, OnSelchangingIndividualTree)
	ON_BN_CLICKED(IDC_SAVE_CHANGES, OnSaveChanges)
	ON_BN_CLICKED(IDC_APPEND_OPENING, OnAppendOpening)
	ON_BN_CLICKED(IDC_APPEND_ROTATION, OnAppendRotation)
	ON_BN_CLICKED(IDC_APPEND_SCALE, OnAppendScale)
	ON_BN_CLICKED(IDC_APPEND_TRANSLATION, OnAppendTranslation)
	ON_BN_CLICKED(IDC_DELETE_COMMAND, OnDeleteCommand)
	ON_BN_CLICKED(IDC_INSERT_OPENING, OnInsertOpening)
	ON_BN_CLICKED(IDC_INSERT_ROTATION, OnInsertRotation)
	ON_BN_CLICKED(IDC_INSERT_SCALE, OnInsertScale)
	ON_BN_CLICKED(IDC_INSERT_TRANSLATION, OnInsertTranslation)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ModifyIndividualDialog message handlers

void ModifyIndividualDialog::OnDone() 
{
	applyChanges();
	if (UpdateData(true)) {
		PopulationProperties::getCurrentDesign()->setRawFitness(m_newFitness);
	}
	int j = GeneticEngine::geneticEngine->getPopulationSize();
	for (int i = 1; i < m_copies; i++) {
		j--;
		if (PopulationProperties::getCurrentDesign() == GeneticEngine::geneticEngine->getPopulation()->getDesign(j)) j--;
		GeneticEngine::geneticEngine->getPopulation()->setDesign(j, new Design(*PopulationProperties::getCurrentDesign()));
	}

	CDialog::OnOK();
}

BOOL ModifyIndividualDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	hideAllEdits();
	initialized = false;

	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	Genotype*	currentGenome = PopulationProperties::getCurrentDesign()->getGenome();

	fillTree(tree, currentGenome);

	m_newFitness = (int)(PopulationProperties::getCurrentDesign()->getRawFitness() * 1000) / 1000.0;
	m_copies = 1;
	UpdateData(false);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void ModifyIndividualDialog::fillTree(CTreeCtrl* tree, Genotype* genome)
{
	for (int i = 0; i < genome->getShapes()->size(); i++) {
		fillPrimary(tree, (*genome->getShapes())[i]);
	}
}

void ModifyIndividualDialog::associateMember(CTreeCtrl* tree, HTREEITEM item, TreeItemType type, PrimaryShape* shape, void* itemPtr)
{
	TreeItem*	  newItem = new TreeItem;

	newItem->itemType = type;
	newItem->shape = shape;
	newItem->itemPtr = itemPtr;

	treeItems.push_back(newItem);
	tree->SetItemData(item, (unsigned long)newItem);

	return;
}

void ModifyIndividualDialog::clearAssociations()
{
	while (! treeItems.empty()) {
		delete *treeItems.begin();
		treeItems.pop_front();
	}
}

void ModifyIndividualDialog::fillPrimary(CTreeCtrl* tree, PrimaryShape* shape)
{
	ostringstream entry;
	HTREEITEM	newItem;

	entry.setf(ios::fixed);
	entry.precision(3);

	HTREEITEM shapeName = tree->InsertItem(shape->getName().c_str());
	associateMember(tree, shapeName, TREE_PRIMARY, shape, NULL);

	HTREEITEM params = tree->InsertItem("Parameters", shapeName);
	associateMember(tree, params, TREE_PARAMETERS, shape, NULL);

	entry << "ZoneType: " << shape->getOriginalPEType().getType();
	newItem = tree->InsertItem(entry.str().c_str(), params);
	entry.str("");
	associateMember(tree, newItem, TREE_PETYPE, shape, NULL);

	entry << "Z-Start: " << shape->getOriginalZStart();
	newItem = tree->InsertItem(entry.str().c_str(), params);
	entry.str("");
	associateMember(tree, newItem, TREE_ZSTART, shape, NULL);

	entry << "Z-Height: " << shape->getOriginalZHeight();
	newItem = tree->InsertItem(entry.str().c_str(), params);
	entry.str("");
	associateMember(tree, newItem, TREE_ZHEIGHT, shape, NULL);

	entry << "Z-Widening: " << shape->getOriginalZWidening();
	newItem = tree->InsertItem(entry.str().c_str(), params);
	entry.str("");
	associateMember(tree, newItem, TREE_ZWIDENING, shape, NULL);

	entry << "Shape mode: " << shape->getOriginalShapeMode();
	newItem = tree->InsertItem(entry.str().c_str(), params);
	entry.str("");
	associateMember(tree, newItem, TREE_SHAPE_MODE, shape, NULL);


	HTREEITEM commHeading = tree->InsertItem("Commands", shapeName);
	associateMember(tree, commHeading, TREE_COMMANDS, shape, NULL);
	commandHeaders.push_back(commHeading);
	fillCommands(tree, commHeading, shape);
}

void ModifyIndividualDialog::fillCommands(CTreeCtrl* tree, HTREEITEM commandHeading, PrimaryShape* shape)
{
	HTREEITEM	newItem;

	const list<Command*>& commands = shape->getOperations();
	for (list<Command*>::const_iterator i= commands.begin(); i != commands.end(); i++) {
		newItem = tree->InsertItem((*i)->print().c_str(), commandHeading);
		switch ((*i)->type()) {
		case COMMAND_OPENING:
			associateMember(tree, newItem, TREE_OPENING, shape, (*i));
			break;
		case COMMAND_REFLECT:
			associateMember(tree, newItem, TREE_REFLECT, shape, (*i));
			break;
		case COMMAND_TRANSLATE:
			associateMember(tree, newItem, TREE_TRANSLATE, shape, (*i));
			break;
		case COMMAND_SCALE:
			associateMember(tree, newItem, TREE_SCALE, shape, (*i));
			break;
		case COMMAND_ROTATE:
			associateMember(tree, newItem, TREE_ROTATE, shape, (*i));
			break;
		}
	}
}

void ModifyIndividualDialog::OnSelchangedIndividualTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	HTREEITEM selected = pNMTreeView->itemNew.hItem;
	selectedItem = selected;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));

	TreeItem* item = (TreeItem*)tree->GetItemData(selected);

	hideAllEdits();
	selectedShape = item->shape;
	selectedShape->MayaSelect();
	eval("refresh");
	selectedCommand = item->itemPtr;
	showPrimaryControls();

	editMode = item->itemType;
	switch (editMode) {
	case TREE_PRIMARY:
		editPrimary(item->shape);
		break;
	case TREE_PARAMETERS:
		editPrimary(item->shape);
		break;
	case TREE_COMMANDS:
		editCommands(item->shape);
		break;
	case TREE_PETYPE:
		editPEType(item->shape);
		break;
	case TREE_ZSTART:
		editZStart(item->shape);
		break;
	case TREE_ZHEIGHT:
		editZHeight(item->shape);
		break;
	case TREE_ZWIDENING:
		editZWidening(item->shape);
		break;
	case TREE_SHAPE_MODE:
		editShapeMode(item->shape);
		break;
	case TREE_OPENING:
		editOpening(item->shape, (Opening*)item->itemPtr);
		break;
	case TREE_REFLECT:
		editReflection(item->shape, (Reflection*)item->itemPtr);
		break;
	case TREE_TRANSLATE:
		editTranslation(item->shape, (Translation*)item->itemPtr);
		break;
	case TREE_SCALE:
		editScale(item->shape, (Scale*)item->itemPtr);
		break;
	case TREE_ROTATE:
		editRotation(item->shape, (Rotation*)item->itemPtr);
		break;
	}
	
	*pResult = 0;
}

void ModifyIndividualDialog::hideAllEdits()
{
	m_edit1 = 1;
	m_edit2 = 1;
	m_edit3 = 1;
	edit1min = 0;
	edit1max = 10;
	edit2min = 0;
	edit2max = 10;
	edit3min = 0;
	edit3max = 10;
	UpdateData(false);
	dataChanged = false;

	GetDlgItem(IDC_APPEND_ROTATION)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_APPEND_SCALE)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_APPEND_TRANSLATION)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_APPEND_OPENING)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_APPEND_COMMAND)->ShowWindow(SW_HIDE);

	GetDlgItem(IDC_INSERT_ROTATION)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_INSERT_SCALE)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_INSERT_TRANSLATION)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_INSERT_OPENING)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_INSERT_COMMAND)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_HIDE);

	GetDlgItem(IDC_EDIT_GROUP)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_LABEL1)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_LABEL2)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_LABEL3)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_EDIT1)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_EDIT2)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_EDIT3)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_RANGE1)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_RANGE2)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_RANGE3)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_SAVE_CHANGES)->ShowWindow(SW_HIDE);

}

void ModifyIndividualDialog::showPrimaryControls()
{
	GetDlgItem(IDC_APPEND_ROTATION)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_APPEND_SCALE)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_APPEND_TRANSLATION)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_APPEND_OPENING)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_APPEND_COMMAND)->ShowWindow(SW_SHOW);
}

void ModifyIndividualDialog::showCommandControls()
{
	GetDlgItem(IDC_INSERT_ROTATION)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_INSERT_SCALE)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_INSERT_TRANSLATION)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_INSERT_OPENING)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_INSERT_COMMAND)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_EDIT_GROUP)->ShowWindow(SW_SHOW);
}

void ModifyIndividualDialog::showEdit1(string label, double value, double min, double max)
{
	edit1min = min;
	edit1max = max;
	m_edit1 = int(value * 1000) / 1000.0;

	SetDlgItemText(IDC_LABEL1, label.c_str());
	GetDlgItem(IDC_LABEL1)->ShowWindow(SW_SHOW);

	GetDlgItem(IDC_EDIT1)->ShowWindow(SW_SHOW);

	ostringstream range;
	range << "From " << min << " to " << max;
	SetDlgItemText(IDC_RANGE1, range.str().c_str());
	GetDlgItem(IDC_RANGE1)->ShowWindow(SW_SHOW);

	UpdateData(false);
}

void ModifyIndividualDialog::showEdit2(string label, double value, double min, double max)
{
	edit2min = min;
	edit2max = max;
	m_edit2 = int(value * 1000) / 1000.0;

	SetDlgItemText(IDC_LABEL2, label.c_str());
	GetDlgItem(IDC_LABEL2)->ShowWindow(SW_SHOW);

	GetDlgItem(IDC_EDIT2)->ShowWindow(SW_SHOW);

	ostringstream range;
	range << "From " << min << " to " << max;
	SetDlgItemText(IDC_RANGE2, range.str().c_str());
	GetDlgItem(IDC_RANGE2)->ShowWindow(SW_SHOW);

	UpdateData(false);
}

void ModifyIndividualDialog::showEdit3(string label, double value, double min, double max)
{
	edit3min = min;
	edit3max = max;
	m_edit3 = int(value * 1000) / 1000.0;

	SetDlgItemText(IDC_LABEL3, label.c_str());
	GetDlgItem(IDC_LABEL3)->ShowWindow(SW_SHOW);

	GetDlgItem(IDC_EDIT3)->ShowWindow(SW_SHOW);

	ostringstream range;
	range << "From " << min << " to " << max;
	SetDlgItemText(IDC_RANGE3, range.str().c_str());
	GetDlgItem(IDC_RANGE3)->ShowWindow(SW_SHOW);

	UpdateData(false);
}

void ModifyIndividualDialog::editPrimary(PrimaryShape* shape)
{
}

void ModifyIndividualDialog::editParameters(PrimaryShape* shape)
{
}

void ModifyIndividualDialog::editCommands(PrimaryShape* shape)
{
}

void ModifyIndividualDialog::editPEType(PrimaryShape* shape)
{
	showEdit1("ZoneType:", shape->getOriginalPEType().getType(), 0, ZoneTypes::zoneTypes->getNumZoneTypes() - 1);
}

void ModifyIndividualDialog::editZStart(PrimaryShape* shape)
{
	showEdit1("ZStart:", shape->getOriginalZStart(), GeneticEngine::getZStartMin(), GeneticEngine::getZStartMax());
}

void ModifyIndividualDialog::editZHeight(PrimaryShape* shape)
{
	showEdit1("ZHeight:", shape->getOriginalZHeight(), GeneticEngine::getZHeightMin(), GeneticEngine::getZHeightMax());
}

void ModifyIndividualDialog::editZWidening(PrimaryShape* shape)
{
	showEdit1("ZWidening:", shape->getOriginalZWidening(), GeneticEngine::getZWideningMin(), GeneticEngine::getZWideningMax());
}

void ModifyIndividualDialog::editShapeMode(PrimaryShape* shape)
{
	showEdit1("Shape Mode:", shape->getOriginalShapeMode(), 1, 45);
}

void ModifyIndividualDialog::editOpening(PrimaryShape* shape, Opening* opening)
{
	showCommandControls();
	showEdit1("Opening start:", opening->getStart(), 0, 1);
	showEdit2("Opening width:", opening->getWidth(), 0, 1);
}

void ModifyIndividualDialog::editReflection(PrimaryShape* shape, Reflection* reflection)
{
	showCommandControls();
	showEdit1("Reflection around:", reflection->getAxes(), 0, 2);
}

void ModifyIndividualDialog::editTranslation(PrimaryShape* shape, Translation* translation)
{
	showCommandControls();
	showEdit1("X:", translation->getX(), Translation::getMinXTranslation(), Translation::getMaxXTranslation());
	showEdit2("Y:", translation->getY(), Translation::getMinYTranslation(), Translation::getMaxYTranslation());
	showEdit3("Z:", translation->getZ(), Translation::getMinZTranslation(), Translation::getMaxZTranslation());
}

void ModifyIndividualDialog::editScale(PrimaryShape* shape, Scale* scale)
{
	showCommandControls();
	showEdit1("X:", scale->getX(), Scale::getMinXScale(), Scale::getMaxXScale());
	showEdit2("Y:", scale->getY(), Scale::getMinYScale(), Scale::getMaxYScale());
	showEdit3("Z:", scale->getZ(), Scale::getMinZScale(), Scale::getMaxZScale());
}

void ModifyIndividualDialog::editRotation(PrimaryShape* shape, Rotation* rotation)
{
	showCommandControls();
	showEdit1("Rotation:", rotation->getRotation(), Rotation::getMinRotation(), Rotation::getMaxRotation());
}

void ModifyIndividualDialog::OnDestroy() 
{
	CDialog::OnDestroy();
	
	clearAssociations();	
}

void ModifyIndividualDialog::OnUpdateEdit1() 
{
	if (! dataChanged) {
		dataChanged = true;
		GetDlgItem(IDC_SAVE_CHANGES)->ShowWindow(SW_SHOW);
//		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_HIDE);
	}
}

void ModifyIndividualDialog::OnUpdateEdit2() 
{

	if (! dataChanged) {
		dataChanged = true;
		GetDlgItem(IDC_SAVE_CHANGES)->ShowWindow(SW_SHOW);
//		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_HIDE);
	}
}

void ModifyIndividualDialog::OnUpdateEdit3() 
{

	if (! dataChanged) {
		dataChanged = true;
		GetDlgItem(IDC_SAVE_CHANGES)->ShowWindow(SW_SHOW);
//		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_HIDE);
	}	
}

bool ModifyIndividualDialog::applyChanges()
{

	if (!initialized) {						// Don't make any changes if nothing has been selected
		initialized = true;
		return true;
	}

	if (! dataChanged) return true;
	if (!UpdateData(true)) return false;

	dataChanged = false;
	ostringstream newText;

	newText.setf(ios::fixed);
	newText.precision(3);

	switch (editMode) {
	case TREE_PRIMARY:
		break;
	case TREE_PARAMETERS:
		break;
	case TREE_COMMANDS:
		break;
	case TREE_PETYPE:
		selectedShape->setOriginalPEType(m_edit1);
		newText << "ZoneType: " << selectedShape->getOriginalPEType().getType();
		break;
	case TREE_ZSTART:
		selectedShape->setOriginalZStart(m_edit1);
		newText << "Z-Start: " << selectedShape->getOriginalZStart();
		break;
	case TREE_ZHEIGHT:
		selectedShape->setOriginalZHeight(m_edit1);
		newText << "Z-Height: " << selectedShape->getOriginalZHeight();
		break;
	case TREE_ZWIDENING:
		selectedShape->setOriginalZWidening(m_edit1);
		newText << "Z-Widening: " << selectedShape->getOriginalZWidening();
		break;
	case TREE_SHAPE_MODE:
		selectedShape->setOriginalShapeMode(m_edit1);
		newText << "Shape mode: " << selectedShape->getOriginalShapeMode();
		break;
	case TREE_OPENING:
		((Opening*)selectedCommand)->setStart(m_edit1);
		((Opening*)selectedCommand)->setWidth(m_edit2);
		newText << ((Opening*)selectedCommand)->print();
		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_SHOW);
		break;
	case TREE_REFLECT:
		if (m_edit1 == 0)
			((Reflection*)selectedCommand)->setAxes(X_REFLECTION);
		else if (m_edit1 == 1)
			((Reflection*)selectedCommand)->setAxes(Y_REFLECTION);
		else
			((Reflection*)selectedCommand)->setAxes(BOTH_REFLECTION);
		newText << ((Reflection*)selectedCommand)->print();
		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_SHOW);
		break;
	case TREE_TRANSLATE:
		((Translation*)selectedCommand)->setX(m_edit1);
		((Translation*)selectedCommand)->setY(m_edit2);
		((Translation*)selectedCommand)->setZ(m_edit3);
		newText << ((Translation*)selectedCommand)->print();
		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_SHOW);
		break;
	case TREE_SCALE:
		((Scale*)selectedCommand)->setX(m_edit1);
		((Scale*)selectedCommand)->setY(m_edit2);
		((Scale*)selectedCommand)->setZ(m_edit3);
		newText << ((Scale*)selectedCommand)->print();
		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_SHOW);
		break;
	case TREE_ROTATE:
		((Rotation*)selectedCommand)->setRotation(m_edit1);
		newText << ((Rotation*)selectedCommand)->print();
		GetDlgItem(IDC_DELETE_COMMAND)->ShowWindow(SW_SHOW);
		break;
	}
	GetDlgItem(IDC_SAVE_CHANGES)->ShowWindow(SW_HIDE);

	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	tree->SetItemText(selectedItem, newText.str().c_str());

	selectedItem = tree->GetSelectedItem();
	TreeItem* item = (TreeItem*)tree->GetItemData(selectedItem);
	selectedShape = item->shape;
	selectedCommand = item->itemPtr;
	editMode = item->itemType;

	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	selectedShape->MayaSelect();
	eval("refresh");

	return true;
}

void ModifyIndividualDialog::OnSelchangingIndividualTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	if (!applyChanges()) {
		*pResult = 1;
		return;
	}

	*pResult = 0;
}

void ModifyIndividualDialog::OnSaveChanges() 
{
	applyChanges();	
}

void ModifyIndividualDialog::OnAppendOpening() 
{
	if (!applyChanges()) return;
	
	Opening*	newOpening = new Opening();
	newOpening->randomize();

	HTREEITEM newItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			newItem = tree->InsertItem(newOpening->print().c_str(), commandHeaders[i]);
			associateMember(tree, newItem, TREE_OPENING, selectedShape, newOpening);
			break;
		}
	}

	selectedShape->appendOperation(newOpening);
	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();
}

void ModifyIndividualDialog::OnAppendRotation() 
{
	if (!applyChanges()) return;
	
	Rotation*	newRotation = new Rotation();
	newRotation->randomize();

	HTREEITEM newItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			newItem = tree->InsertItem(newRotation->print().c_str(), commandHeaders[i]);
			associateMember(tree, newItem, TREE_ROTATE, selectedShape, newRotation);
			break;
		}
	}

	selectedShape->appendOperation(newRotation);
	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();
}

void ModifyIndividualDialog::OnAppendScale() 
{
	if (!applyChanges()) return;
	
	Scale*	newScale = new Scale();
	newScale->randomize();

	HTREEITEM newItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			newItem = tree->InsertItem(newScale->print().c_str(), commandHeaders[i]);
			associateMember(tree, newItem, TREE_SCALE, selectedShape, newScale);
			break;
		}
	}

	selectedShape->appendOperation(newScale);
	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();
}

void ModifyIndividualDialog::OnAppendTranslation() 
{
	if (!applyChanges()) return;
	
	Translation*	newTranslation = new Translation();
	newTranslation->randomize();

	HTREEITEM newItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			newItem = tree->InsertItem(newTranslation->print().c_str(), commandHeaders[i]);
			associateMember(tree, newItem, TREE_TRANSLATE, selectedShape, newTranslation);
			break;
		}
	}

	selectedShape->appendOperation(newTranslation);
	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();	
}

void ModifyIndividualDialog::OnDeleteCommand() 
{
	dataChanged = false;

	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	HTREEITEM oldItem = selectedItem;

	selectedShape->removeOperation((Command*)selectedCommand);

	HTREEITEM prevItem = tree->GetNextItem( oldItem, TVGN_PREVIOUS );
	if (! prevItem)
		prevItem = tree->GetNextItem( oldItem, TVGN_PREVIOUSVISIBLE );
	if (prevItem)
		tree->SelectItem(prevItem);
	tree->DeleteItem(oldItem);

	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");
}

void ModifyIndividualDialog::OnInsertOpening() 
{
	if (!applyChanges()) return;
	
	Opening*	newOpening = new Opening();
	newOpening->randomize();

	HTREEITEM newItem, prevItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	prevItem = tree->GetPrevVisibleItem( selectedItem );

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			if (prevItem == commandHeaders[i]) {
				newItem = tree->InsertItem(newOpening->print().c_str(), commandHeaders[i], TVI_FIRST);
			}
			else {
				newItem = tree->InsertItem(newOpening->print().c_str(), commandHeaders[i], prevItem);
			}
			associateMember(tree, newItem, TREE_OPENING, selectedShape, newOpening);
			break;
		}
	}

	selectedShape->insertOperationBefore((Command*)selectedCommand, newOpening);

	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();	
}

void ModifyIndividualDialog::OnInsertRotation() 
{
	if (!applyChanges()) return;
	
	Rotation*	newRotation = new Rotation();
	newRotation->randomize();

	HTREEITEM newItem, prevItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	prevItem = tree->GetPrevVisibleItem( selectedItem );

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			if (prevItem == commandHeaders[i]) {
				newItem = tree->InsertItem(newRotation->print().c_str(), commandHeaders[i], TVI_FIRST);
			}
			else {
				newItem = tree->InsertItem(newRotation->print().c_str(), commandHeaders[i], prevItem);
			}
			associateMember(tree, newItem, TREE_ROTATE, selectedShape, newRotation);
			break;
		}
	}

	selectedShape->insertOperationBefore((Command*)selectedCommand, newRotation);

	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();	
}

void ModifyIndividualDialog::OnInsertScale() 
{
	if (!applyChanges()) return;
	
	Scale*	newScale = new Scale();
	newScale->randomize();

	HTREEITEM newItem, prevItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	prevItem = tree->GetPrevVisibleItem( selectedItem );

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			if (prevItem == commandHeaders[i]) {
				newItem = tree->InsertItem(newScale->print().c_str(), commandHeaders[i], TVI_FIRST);
			}
			else {
				newItem = tree->InsertItem(newScale->print().c_str(), commandHeaders[i], prevItem);
			}
			associateMember(tree, newItem, TREE_SCALE, selectedShape, newScale);
			break;
		}
	}

	selectedShape->insertOperationBefore((Command*)selectedCommand, newScale);

	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();	
}

void ModifyIndividualDialog::OnInsertTranslation() 
{
	if (!applyChanges()) return;
	
	Translation*	newTranslation = new Translation();
	newTranslation->randomize();

	HTREEITEM newItem, prevItem;
	CTreeCtrl* tree = ((CTreeCtrl*)GetDlgItem(IDC_INDIVIDUAL_TREE));
	prevItem = tree->GetPrevVisibleItem( selectedItem );

	for (int i = 0; i < commandHeaders.size(); i++) {
		if ( ((TreeItem*)tree->GetItemData(commandHeaders[i]))->shape == selectedShape ) {
			if (prevItem == commandHeaders[i]) {
				newItem = tree->InsertItem(newTranslation->print().c_str(), commandHeaders[i], TVI_FIRST);
			}
			else {
				newItem = tree->InsertItem(newTranslation->print().c_str(), commandHeaders[i], prevItem);
			}
			associateMember(tree, newItem, TREE_TRANSLATE, selectedShape, newTranslation);
			break;
		}
	}

	selectedShape->insertOperationBefore((Command*)selectedCommand, newTranslation);

	PopulationProperties::getCurrentDesign()->erase();
	PopulationProperties::getCurrentDesign()->interpret();
	eval("refresh");

	tree->SelectItem(newItem);
	GetDlgItem(IDC_EDIT1)->SetFocus();	
}
