//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
#if !defined(AFX_MODIFYINDIVIDUALDIALOG_H__8163DCE3_4C2F_11D4_9F92_0000E87A7A07__INCLUDED_)
#define AFX_MODIFYINDIVIDUALDIALOG_H__8163DCE3_4C2F_11D4_9F92_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ModifyIndividualDialog.h : header file
//

#include "Genotype.h"
#include "PrimaryShape.h"
#include "Command.h"
#include "Opening.h"
#include "Reflection.h"
#include "Rotation.h"
#include "Scale.h"
#include "Translation.h"
#include <list>

typedef enum {TREE_PRIMARY, TREE_PARAMETERS, TREE_COMMANDS, TREE_PETYPE, TREE_ZSTART, TREE_ZHEIGHT, 
			TREE_ZWIDENING, TREE_SHAPE_MODE, TREE_OPENING, TREE_REFLECT, TREE_TRANSLATE, TREE_SCALE, TREE_ROTATE} TreeItemType;

typedef struct {
	TreeItemType	itemType;
	PrimaryShape*	shape;
	void*			itemPtr;
}	TreeItem;

using namespace std;

/////////////////////////////////////////////////////////////////////////////
// ModifyIndividualDialog dialog

class ModifyIndividualDialog : public CDialog
{
// Construction
public:
	ModifyIndividualDialog(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(ModifyIndividualDialog)
	enum { IDD = IDD_MODIFY_INDIVIDUAL_DIALOG };
	double	m_edit1;
	double	m_edit2;
	double	m_edit3;
	double	m_newFitness;
	int		m_copies;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(ModifyIndividualDialog)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(ModifyIndividualDialog)
	afx_msg void OnDone();
	virtual BOOL OnInitDialog();
	afx_msg void OnSelchangedIndividualTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDestroy();
	afx_msg void OnUpdateEdit1();
	afx_msg void OnUpdateEdit2();
	afx_msg void OnUpdateEdit3();
	afx_msg void OnSelchangingIndividualTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSaveChanges();
	afx_msg void OnAppendOpening();
	afx_msg void OnAppendRotation();
	afx_msg void OnAppendScale();
	afx_msg void OnAppendTranslation();
	afx_msg void OnDeleteCommand();
	afx_msg void OnInsertOpening();
	afx_msg void OnInsertRotation();
	afx_msg void OnInsertScale();
	afx_msg void OnInsertTranslation();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:

	void fillTree(CTreeCtrl* tree, Genotype* genome);
	void fillPrimary(CTreeCtrl* tree, PrimaryShape* shape);
	void fillCommands(CTreeCtrl* tree, HTREEITEM commandHeading, PrimaryShape* shape);
	void associateMember(CTreeCtrl* tree, HTREEITEM item, TreeItemType type, PrimaryShape* shape, void* itemPtr);
	void clearAssociations();

// Window preparation functions:

	void hideAllEdits();
	void editPrimary(PrimaryShape* shape);
	void editParameters(PrimaryShape* shape);
	void editCommands(PrimaryShape* shape);
	void editPEType(PrimaryShape* shape);
	void editZStart(PrimaryShape* shape);
	void editZHeight(PrimaryShape* shape);
	void editZWidening(PrimaryShape* shape);
	void editShapeMode(PrimaryShape* shape);
	void editOpening(PrimaryShape* shape, Opening* opening);
	void editReflection(PrimaryShape* shape, Reflection* reflection);
	void editTranslation(PrimaryShape* shape, Translation* translation);
	void editScale(PrimaryShape* shape, Scale* scale);
	void editRotation(PrimaryShape* shape, Rotation* rotation);
	void showPrimaryControls();
	void showCommandControls();
	void showEdit1(string label, double value, double min, double max);
	void showEdit2(string label, double value, double min, double max);
	void showEdit3(string label, double value, double min, double max);
	bool applyChanges();

// Data members
	bool					initialized;
	bool					dataChanged;
	TreeItemType			editMode;
	list<TreeItem*>			treeItems;
	PrimaryShape*			selectedShape;
	void*					selectedCommand;
	HTREEITEM				selectedItem;
	vector<HTREEITEM>		commandHeaders;

	double	edit1min, edit1max;
	double	edit2min, edit2max;
	double	edit3min, edit3max;

};
//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MODIFYINDIVIDUALDIALOG_H__8163DCE3_4C2F_11D4_9F92_0000E87A7A07__INCLUDED_)
