//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Phenotype.h: interface for the Phenotype class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PHENOTYPE_H__632786DD_ED45_11D3_AC39_00C04F797C24__INCLUDED_)
#define AFX_PHENOTYPE_H__632786DD_ED45_11D3_AC39_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <list>
#include <vector>
#include <string>
#include "ShapePair.h"

class Genotype;
class Shape;

using namespace std;

const string LAYER_NAME = "DesignLayer";

typedef struct {
	Shape*	shape;
	Shape*	parent;
} Slice;

class Phenotype  
{
public:
	void makeImpressionsInvalid();
	bool getImpression(int AgentType, int ZoneType, double& impression);
	void storeImpression(int agentTypeNum, int zoneTypeNum, double impression);
	void initializeImpressionMatrix();
	void makeImpressionsValid()	{_impressionsValid = true;};
	void hideLayer();
	Phenotype(Genotype* genome = NULL);
	Phenotype(const Phenotype& oldPhenotype);
	virtual ~Phenotype();

	const Phenotype& operator=(const Phenotype& oldPhenotype);

// Accessors:
	bool			updated() {return _updated;};			// Is this phenotype up to date?
	list<Shape*>&	getShapes() {return _shapes;};

// Mutators
	
	void	needsUpdate() {_updated = false;};
	void	setShapes(vector<Shape*>* newShapes);
	void	setGenome(Genotype* genome);
	void	clearShapes();
	void	setEraseOnDelete(bool newErase);

	// Maya construction / deconstruction
	Phenotype&	interpret(bool startFromZero = false);
	void runPrimaryCommands(bool startFromZero = false);
	void erase();

	void processIntersections(bool startFromZero = false);
	void findIntersections(list<Shape*>& shapeList, list<ShapePair*>& ipairs);
	void removeIntersections(list<ShapePair*>& ipairs, list<Shape*>& shapeList);
	void sliceIntersections(list<ShapePair*>& ipairs, list<Shape*>& nonIntersecting, list<ShapePair*>& fullyIntersecting);				// Slice all intersections to produce only shapes that intersect nothing and that are fully intersected
	void performBooleans(list<ShapePair*>& fullyIntersecting, list<Shape*>& booleanShapes);
	void fillAllIntersections(list<ShapePair*>& ipairs, list<list<Shape*>*>& allIntersections);
	void julienne(list<Shape*>& intersections, list<Slice*>& slices);

	string&	getLayerName()	{return _layerName;};
	void	setLayerName(const string layer)	{_layerName = layer;};

private:
	list<Shape*> _shapes;
	Genotype*	 _genome;
	bool		 _updated;
	bool		 _eraseOnDelete;
	list<Shape*> _eraseList;
	string		 _layerName;
	vector<vector<double>*>*	_impressions;	//Per AgentType per ZoneType

	string	getNewLayerName();
protected:
	bool _impressionsValid;
};

#endif // !defined(AFX_PHENOTYPE_H__632786DD_ED45_11D3_AC39_00C04F797C24__INCLUDED_)
