//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
#include "stdafx.h"
#include <maya/MSimple.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MDagPath.h>
#include <maya/MFnDagNode.h>
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
#include <iostream>
#include <time.h>
#include "Random.h"
#include "PopulationProperties.h"
#include "GeneticEngineProperties.h"
#include "GeneticEngine.h"
#include "Constants.h"
#include "Resource.h"
#include "AgentControl.h"
#include "MELUtilities.h"
#include "MayaUtilities.h"
#include "MyPropertySheet.h"
#include "AgentProperties.h"
#include "SettingsProps.h"
#include "Settings.h"
#include "Design.h"
#include "ZoneTypes.h"
#include "ZoneTypeProperties.h"
#include "AboutProperties.h"
#include "MELUtilities.h"
#include <sstream>
#include "Layer.h"
#include "afxmt.h"
#include "Miscellaneous.h"

GeneticEngine* initializeEngine();		// Templatize function below
DWORD WINAPI EngineThread(LPVOID pParam);
CEvent EndEvent;



DeclareSimpleCommand( agency, "Alias|Wavefront", "1.1");

MStatus agency::doIt( const MArgList& )
{



	cout << endl << endl << "Starting AgencyGP..." << endl;
	
	//AfxBeginThread(EngineThread, NULL, THREAD_PRIORITY_NORMAL);
	//HANDLE threadHandle = CreateThread(NULL, 0, EngineThread, 0, 0, NULL);

// Initialization routines

	
	
	
	ostringstream command;
	AFX_MANAGE_STATE(AfxGetStaticModuleState( ));  // Necessary to use MFC classes
	MString result;
	MGlobal::executeCommand("getenv \"MAYA_LOCATION\"", result);
	string mayaLoc = result.asChar();
	for (int i = 0; i < mayaLoc.size(); i++)
		if (mayaLoc[i] == '\\') mayaLoc[i] = '/';

	command << "source \"" << mayaLoc << "/bin/plug-ins/Agency.mel\"";
	eval(command.str());


// Actual code
    

	eval("editDisplayLayerGlobals -useCurrent on");		// Set up MAYA global preferences

	
	Settings theSettings;
	theSettings.
	Settings::readSettings();
	ZoneTypes zoneTypes;
	Layer layers;
	cout << "Initializing engine" << endl;
	GeneticEngine* theEngine = initializeEngine();
	zoneTypes.setEngine(theEngine);

	cout << "Initialized engine." << endl;

	


	if (theEngine != NULL) {
		
		Design starter;
		AgentControl*	theControlCenter = new AgentControl(&starter);
		zoneTypes.setAgentControl(theControlCenter);

		Settings::readSettings();

		MyPropertySheet mainProps("Agency GP");
		AgentProperties agentProps;
		agentProps.setEngine(theEngine);
		PopulationProperties popProps;
		popProps.setEngine(theEngine);
		theEngine->setPopProps(&popProps);

		GeneticEngineProperties geneticProps;
		geneticProps.setEngine(theEngine);

		SettingsProps settingsProps;
		ZoneTypeProperties zoneTypeProps;

		Miscellaneous miscPage;
		AboutProperties aboutPage;

		mainProps.AddPage(&popProps);
		mainProps.AddPage(&agentProps);
		mainProps.AddPage(&zoneTypeProps);
		mainProps.AddPage(&geneticProps);
		mainProps.AddPage(&settingsProps);
		mainProps.AddPage(&miscPage);
		mainProps.AddPage(&aboutPage);

		mainProps.DoModal();
		
		cout << "Deleting engine." << endl;
		
		delete theEngine;
	}
	else {
		cout << "Engine initialize failed." << endl;
	}
	//cout << "Waiting for thread." << endl;
	//DWORD wait = WaitForSingleObjectEx(threadHandle, INFINITE, TRUE);
	//if(wait==WAIT_FAILED)
	//	cout << "Error: " << GetLastError() << endl;
    return MS::kSuccess;
}

GeneticEngine* initializeEngine() {

	GeneticEngine* theEngine = NULL;
	MSelectionList      list;
	MStringArray		names;
	std::vector<string> shapeNames;
	std::vector<string> layerNames;

	int selCount = 0;
    MGlobal::getActiveSelectionList( list );
	for ( MItSelectionList listIter( list, MFn::kNurbsCurve ); !listIter.isDone(); listIter.next() )
	{
		selCount++;
	}

	if (selCount == 0) {
		AfxMessageBox("You must select at least one NURBS curve for AgencyGP to work.");
		return NULL;
	}

// This code gets the Maya curves selected and closes them

	shapeNames.resize(selCount);
	layerNames.resize(selCount);
	int i = 0;
	string layerName;
	for ( listIter.reset(); !listIter.isDone(); listIter.next(), i++)
	{
		listIter.getStrings(names);
		shapeNames[i] = (names[0]).asChar();
		eval("int $isClosed = `getAttr \"" + shapeNames[i] + ".form\"`");
		if (getMELint("$isClosed") < 2)
			eval("closeCurve -ch off -rpo on " + shapeNames[i]);

		eval("string $layername = `objectLayer " + shapeNames[i] + "`");
		getMELvalue("$layername", layerName);
		Layer::layer->addLayer(layerName);
		layerNames[i] = layerName;
	}

	initializeShaderGroups();
	ZoneTypes::zoneTypes->initializeZoneTypes();
	
	theEngine = new GeneticEngine(POPULATION_SIZE, shapeNames, layerNames, &list);

	if (theEngine == NULL) AfxMessageBox("Genetic engine could not be initialized.");

	return theEngine;
}

DWORD WINAPI EngineThread(LPVOID pParam)
{
	ostringstream command;
	AFX_MANAGE_STATE(AfxGetStaticModuleState( ));  // Necessary to use MFC classes

// Initialization routines

		
	MString result;
	MGlobal::executeCommand("getenv \"MAYA_LOCATION\"", result);
	string mayaLoc = result.asChar();
	for (int i = 0; i < mayaLoc.size(); i++)
		if (mayaLoc[i] == '\\') mayaLoc[i] = '/';

	command << "source \"" << mayaLoc << "/bin/plug-ins/Agency.mel\"";
	eval(command.str());
//	eval("source \"C:/aw/maya3.0/bin/plug-ins/Agency.mel\"");
/*	getMELvalue("$noAgency", noAgency);
	if (noAgency) {
		AfxMessageBox("File \"Agency.mel\" not found in plug-ins directory.");
		return MS::kSuccess;
	}*/
// Must make it stop if not found


	//time_t seed;
	//time( &seed );
	//Random::sgenrand(seed);

// Actual code
    

	eval("editDisplayLayerGlobals -useCurrent on");		// Set up MAYA global preferences

	Settings theSettings;
	Settings::readSettings();
	ZoneTypes zoneTypes;
	Layer layers;
	cout << "Initializing engine" << endl;
	GeneticEngine* theEngine = initializeEngine();
	zoneTypes.setEngine(theEngine);

	cout << "Initialized engine." << endl;

	


	if (theEngine != NULL) {
		
		Design starter;
		AgentControl*	theControlCenter = new AgentControl(&starter);
		zoneTypes.setAgentControl(theControlCenter);

		Settings::readSettings();

		MyPropertySheet mainProps("Agency GP");
		AgentProperties agentProps;
		agentProps.setEngine(theEngine);
		PopulationProperties popProps;
		popProps.setEngine(theEngine);
		theEngine->setPopProps(&popProps);

		GeneticEngineProperties geneticProps;
		geneticProps.setEngine(theEngine);

		SettingsProps settingsProps;
		ZoneTypeProperties zoneTypeProps;

		AboutProperties aboutPage;

		mainProps.AddPage(&popProps);
		mainProps.AddPage(&agentProps);
		mainProps.AddPage(&zoneTypeProps);
		mainProps.AddPage(&geneticProps);
		mainProps.AddPage(&settingsProps);
		mainProps.AddPage(&aboutPage);

		mainProps.DoModal();
		
		cout << "Deleting engine." << endl;
		
		delete theEngine;
		cout << "Deleted engine." << endl;
	}
	else {
		cout << "Engine initialize failed." << endl;
	}
	cout << "Returning" << endl;
	//EndEvent.PulseEvent();
	//::PostQuitMessage(0);
	ExitThread(0);
	return 0;
}

LRESULT OnAGENCYMess( WPARAM wParam, LPARAM lParam )
{
	return MS::kSuccess;
}