//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Population.cpp: implementation of the Population class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "Population.h"
#include "Design.h"
#include "MayaInfo.h"

#include "Constants.h"
#include <iostream>
#include <list>
#include <algorithm>

class CompareFitness {
public:
	int operator() (Design* d1, Design* d2)
	{return (d1->getRawFitness() < d2->getRawFitness());}
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Population::Population()
{
	designs = new std::vector<Design*>();
}

Population::Population(int size) {

	designs = new std::vector<Design*>(size);
	for(int i = 0; i < size; i++) {
		(*designs)[i] = new Design();
	}
}

/*Population::Population(int size, int shapesPerDesign) {

	designs = new std::vector<Design*>(size);
	for(int i = 0; i < size; i++) {
		(*designs)[i] = new Design(shapesPerDesign);
	}
}*/

Population::Population(int size, const std::vector<string>& shapeNames, const std::vector<string>& layerNames) {
	designs = new std::vector<Design*>(size);
	originalCurves = new std::vector<MayaInfo*>(shapeNames.size());

	for (int i = 0; i < shapeNames.size(); i++)
		(*originalCurves)[i] = new MayaInfo(shapeNames[i]);

	for(i = 0; i < size; i++) {
		(*designs)[i] = new Design(*originalCurves, layerNames);
	}
}

Population::Population(Population& oldPopulation) {

	designs = new std::vector<Design*>(oldPopulation.getDesigns()->size());
	for(int i = 0; i < designs->size(); i++) {
		(*designs)[i] = new Design(*(oldPopulation.getDesign(i)));  // Does a whole member-wise copy, VERY EXPENSIVE
	}
}

Population::~Population()
{
	//designs are deleted
	clearDesigns();
	if (originalCurves) {
		for (int i = 0; i < originalCurves->size(); i++) {
			delete (*originalCurves)[i];
		}
	}
}

void Population::traceOriginalShapeNames() const{
	for (int i = 0; i < originalCurves->size(); i++)
		cout << "Original shape [" << i << "]: " << ((*originalCurves)[i])->name() << endl;

}

void Population::clearDesigns() {

	if (TRACE_LEVEL > 9) cout << "clearDesigns 1" << endl;

	for (int i = 0; i < designs->size(); i++) {
		if (TRACE_LEVEL > 9) cout << "clearDesigns 2, " << i << endl;
		delete (*designs)[i];
		if (TRACE_LEVEL > 9) cout << "clearDesigns 3, " << i << endl;
	}

	delete designs;
	if (TRACE_LEVEL > 9) cout << "clearDesigns 4" << endl;
}

void Population::setDesigns(std::vector<Design*>* newPop) {

	if (TRACE_LEVEL > 9) cout << "Into setDesigns." << endl;


	clearDesigns();
	
	if (TRACE_LEVEL > 9) cout << "Out of clearDesigns." << endl;

	designs = newPop;

	if (TRACE_LEVEL > 9) cout << "Leaving setDesigns." << endl;
}


void Population::setDesign(int index, Design* newDesign) {
	if (index >= 0 && index < designs->size()) {
		delete (*designs)[index];
		(*designs)[index] = newDesign;
	}
}

void Population::randomize(int minNumberOfStartingTransformations, int maxNumberOfStartingTransformations) {
	for(int i = 0; i < designs->size(); i++) {
		(*designs)[i]->randomize(minNumberOfStartingTransformations, maxNumberOfStartingTransformations);
	}
}

const Population& Population::operator=(const Population& oldPopulation) {
	
	clearDesigns();

	designs = new std::vector<Design*>(oldPopulation.getDesigns()->size());
	for(int i = 0; i < designs->size(); i++) {
		(*designs)[i] = new Design(*(oldPopulation.getDesign(i)));  // Does a whole member-wise copy, VERY EXPENSIVE
	}

	return oldPopulation;
}


void Population::clearFitnesses()
{	
	for(int i = 0; i < designs->size(); i++) {
		(*designs)[i]->setFitnessUpdated(false);
	}
}

void Population::sort()
{
	std::sort(designs->begin(), designs->end(), CompareFitness());
}

