//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Population.h: interface for the Population class.
//
// Defines the structure that holds a population of individuals,
// Each of which is stored in a Design (which are composed 
// of a Phenotype and a Genotype, etc.) 
// Most functionality is stored in the object contained by a Population,
// not in the Population class itself
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_POPULATION_H__B350BB99_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
#define AFX_POPULATION_H__B350BB99_E931_11D3_9F59_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include <string>
#include "MayaInfo.h"
#include "Design.h"

using namespace std;

class Population  
{
public:
	Population();
	Population(int size); //create a population of a specified size (of empty designs) - important since there is no other easy way to specify the size of a population - not a problem since populations should not change size
//	Population(int size, int shapesPerDesign); //create a population of a specified size of designs with the a specified number of primary shapes
	Population(int size, const std::vector<string>& shapeNames, const std::vector<string>& layerNames); // Give each primary shape its proper name
	Population(Population& oldPopulation);
	virtual ~Population();

// Accessors
	inline std::vector<Design*>* getDesigns() const {return designs;}; //return the actual vector of Design* that constitutes the population (possibly inefficient - should this be a reference?)
	inline Design* getDesign(int index) const {return ((*designs)[index]);}; //return a pointer to a selected Design

// Mutators
	void setDesigns(std::vector<Design*>* newPop); //set the vector of Design* that consitute the population, pointer to good persistent vector
	void setDesign(int index, Design* newDesign);//reset a single design in the vector of Designs that make up the population, pointer good and persistent
	void randomize(int minNumberOfStartingTransformations, int maxNumberOfStartingTransformations); //set all of the parameters and the command list of each design to random values (in order to initialize the population)

// Utility
	const Population& operator=(const Population& oldPopulation);	//defines member based copying through the equal operator
	void clearDesigns();											// Clears and deletes designs vector
	void clearFitnesses();
	void sort();

// Trace
	void traceOriginalShapeNames() const;  // Dump the names of all original shapes


private:

	std::vector<Design*>* designs; //the set of designs that comprise a population
	std::vector<MayaInfo*>* originalCurves;
};



#endif // !defined(AFX_POPULATION_H__B350BB99_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
