//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// PopulationProperties.cpp : implementation file
//

#include "stdafx.h"
#include "MFCPlugin.h"
#include "PopulationProperties.h"

#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MDagPath.h>
#include <maya/MFnDagNode.h>
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
#include <maya/MFnTransform.h>
#include "Utilities.h"
#include "MELUtilities.h"
#include <iostream>
#include <sstream>
#include <time.h>
#include "Random.h"
#include "Constants.h"
#include "GeneticEngine.h"
#include "ModifyIndividualDialog.h"
#include "Settings.h"
#include <afxmt.h>
#include "MayaUtilities.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ON_UPDATE_MESS WM_USER + 300
#define REFRESH ON_UPDATE_MESS + 1
#define RESTORE_BUTTONS ON_UPDATE_MESS + 2
#define ERASE ON_UPDATE_MESS + 3


/////////////////////////////////////////////////////////////////////////////
// PopulationProperties property page

IMPLEMENT_DYNCREATE(PopulationProperties, CPropertyPage)

// Statics
GeneticEngine* PopulationProperties::theEngine = NULL;
int PopulationProperties::currentDesign = 0;
PopulationProperties* PopulationProperties::thePopPropsPage = NULL;

//To allow the window to communicate with the worker thread we use a shared bool protected by semaphores.
//Not very elegant, but it works.
bool bBreak;


PopulationProperties::PopulationProperties() : CPropertyPage(PopulationProperties::IDD)
{
	//{{AFX_DATA_INIT(PopulationProperties)
	m_nGenerations = 0;
	//}}AFX_DATA_INIT

	theEngine = NULL;
	currentDesign = 0;
	thePopPropsPage = this;
	_taskUnderway = false;
}

PopulationProperties::~PopulationProperties()
{
}

void PopulationProperties::updateDialogText() {
	ostringstream genStr, popStr, meanStr, devStr, indStr, fitStr;

	genStr.setf(ios::fixed);
	genStr.precision(3);

	meanStr.setf(ios::fixed);
	meanStr.precision(3);

	devStr.setf(ios::fixed);
	devStr.precision(3);

	fitStr.setf(ios::fixed);
	fitStr.precision(3);

	genStr << "Generation: " << theEngine->getGenerationsRun();
	SetDlgItemText(IDC_GENERATION, genStr.str().c_str());

	popStr << "Population Size: " << theEngine->getPopulationSize();
	SetDlgItemText(IDC_POP_SIZE, popStr.str().c_str());

	meanStr << "Mean Unfitness: " << theEngine->getMeanFitness();
	SetDlgItemText(IDC_MEAN_FITNESS, meanStr.str().c_str());

	devStr << "Standard Deviation: " << theEngine->getStdDevFitness();
	SetDlgItemText(IDC_STD_DEV, devStr.str().c_str());

	indStr << "Individual: " << currentDesign + 1;
	SetDlgItemText(IDC_INDIVIDUAL, indStr.str().c_str());
	
	fitStr << "Unfitness: " << theEngine->getPopulation()->getDesign(currentDesign)->getRawFitness();
	SetDlgItemText(IDC_FITNESS, fitStr.str().c_str());
}

void PopulationProperties::updateBreedingText() {
	CEdit* editBox = (CEdit*)GetDlgItem(IDC_EDIT1);
	CString oldText;
	string newText;

	editBox->GetWindowText(oldText);
	theEngine->returnStats(newText);
	editBox->SetWindowText(oldText + newText.c_str());
	editBox->LineScroll(editBox->GetLineCount() - 1);
}
void PopulationProperties::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(PopulationProperties)
	DDX_Text(pDX, IDC_N_GENERATIONS_EDIT, m_nGenerations);
	DDV_MinMaxInt(pDX, m_nGenerations, 1, 10000);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(PopulationProperties, CPropertyPage)
	//{{AFX_MSG_MAP(PopulationProperties)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SHIFT_INDIVIDUAL, OnDeltaposShiftIndividual)
	ON_CBN_SELENDOK(IDC_SELECT_INDIVIDUAL, OnSelendokSelectIndividual)
	ON_BN_CLICKED(IDC_STARTGP, OnStartgp)
	ON_BN_CLICKED(IDC_RUN_N_GENERATIONS, OnRunNGenerations)
	ON_BN_CLICKED(IDC_MODIFY_INDIVIDUAL_BUTTON, OnModifyIndividualButton)
	ON_BN_CLICKED(IDC_SAVE_INDIVIDUAL, OnSaveIndividual)
	ON_BN_CLICKED(IDC_SAVE_POPULATION, OnSavePopulation)
	//}}AFX_MSG_MAP
	ON_MESSAGE(ON_UPDATE_MESS, OnUpdateMess)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PopulationProperties message handlers

void PopulationProperties::OnOK() 
{
	// TODO: Add your specialized code here and/or call the base class
	//theEngine->getPopulation()->getDesign(currentDesign)->setEraseOnDelete(false);
	string defaultLayer = "Default";
	getCurrentDesign()->saveDesign(defaultLayer);
	eval("delete Default");
	CPropertyPage::OnOK();
}

void PopulationProperties::interpret() {

	//setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	setLayerVisibility(getCurrentDesign()->getPhenome().getLayerName(), true);
	setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	//theEngine->getPopulation()->getDesign(currentDesign)->interpret(false);
	eval("refresh");
}

BOOL PopulationProperties::OnInitDialog() 
{

	receiveSettings();

	CPropertyPage::OnInitDialog();

//	theEngine->getPopulation()->traceOriginalShapeNames();

	char buffer[20];
	
	CComboBox* indList = (CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL);
	indList->ResetContent();
	for (int i = 0; i < theEngine->getPopulationSize(); i++) {
		itoa(i+1, buffer, 10);
		indList->AddString(buffer);
	}
	indList->SetCurSel(0);

	oldPopSize = theEngine->getPopulationSize();
	_initialized = false;

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void PopulationProperties::OnDeltaposShiftIndividual(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	CSpinButtonCtrl* spin = (CSpinButtonCtrl*)GetDlgItem(IDC_SHIFT_INDIVIDUAL);
	CComboBox* indList = (CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL);

	int curPos = indList->GetCurSel();

	if ((curPos - pNMUpDown->iDelta) < 0) return;
	if ((curPos - pNMUpDown->iDelta) > (theEngine->getPopulationSize() - 1)) return;

	indList->SetCurSel(curPos - pNMUpDown->iDelta);
	OnSelendokSelectIndividual();

	*pResult = 0;
}

void PopulationProperties::changeCurrent(int newCurrent)
{
	//theEngine->getPopulation()->getDesign(currentDesign)->erase();
	getCurrentDesign()->getPhenome().hideLayer();
	currentDesign = newCurrent;
	updateDialogText();
	//Switch layer
	//setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	setLayerVisibility(getCurrentDesign()->getPhenome().getLayerName(), true);
	setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	eval("refresh");
	//interpret();
}

void PopulationProperties::OnSelendokSelectIndividual()
{
	//theEngine->getPopulation()->getDesign(currentDesign)->erase();
	setLayerVisibility(getCurrentDesign()->getPhenome().getLayerName(), false);
	currentDesign = ((CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL))->GetCurSel();
	updateDialogText();
	//Switch layer
	//setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	setLayerVisibility(getCurrentDesign()->getPhenome().getLayerName(), true);
	setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	eval("refresh");
	//interpret();
}

void PopulationProperties::OnCancel() 
{
	//remove all the layers
	theEngine->getPopulation()->getDesign(currentDesign)->erase();	
	CPropertyPage::OnCancel();
}

void PopulationProperties::OnStartgp() 
{

	newTask("Evaluating new population...", theEngine->getPopulationSize());

	if (! _initialized) {
		if(!theEngine->isSeeded())	{
			time_t seed;
			time( &seed );
			Random::sgenrand(seed);
			theEngine->SeedEngine();
		}
		theEngine->seedInitialPopulation();
		theEngine->getPopulation()->sort();
		updateDialogText();
		
		interpret();
		((CButton*)GetDlgItem(IDC_STARTGP))->SetWindowText("Next Generation");

		GetDlgItem(IDC_SAVE_POPULATION)->EnableWindow(true);
		GetDlgItem(IDC_RUN_N_GENERATIONS)->EnableWindow(true);
		GetDlgItem(IDC_N_GENERATIONS_EDIT)->EnableWindow(true);
		GetDlgItem(IDC_SELECT_INDIVIDUAL)->EnableWindow(true);
		GetDlgItem(IDC_SHIFT_INDIVIDUAL)->EnableWindow(true);
		GetDlgItem(IDC_MODIFY_INDIVIDUAL_BUTTON)->EnableWindow(true);
		GetDlgItem(IDC_SAVE_INDIVIDUAL)->EnableWindow(true);
		_initialized = true;
	}
	else {
		//remove all the layers
		theEngine->getPopulation()->getDesign(currentDesign)->erase();

		theEngine->runPopulation(1);
		updateDialogText();
		updateBreedingText();
		interpret();

	}
	taskFinished();
}

void PopulationProperties::setEngine(GeneticEngine* newEngine) {
	theEngine = newEngine;
}

BOOL PopulationProperties::OnSetActive()
{

	if (!CPropertyPage::OnSetActive()) return false;

	if (theEngine->getPopulationSize() != oldPopSize) {

		char buffer[20];
	
		CComboBox* indList = (CComboBox*)GetDlgItem(IDC_SELECT_INDIVIDUAL);
		indList->ResetContent();
		for (int i = 0; i < theEngine->getPopulationSize(); i++) {
			itoa(i+1, buffer, 10);
			indList->AddString(buffer);
		}

//		if (currentDesign >= theEngine->getPopulationSize()) {
			indList->SetCurSel(0);
			currentDesign = 0;
			if (_initialized) OnSelendokSelectIndividual();
//		}
//		else {
//			indList->SetCurSel(currentDesign);
//			if (_initialized) OnSelendokSelectIndividual();
//		}

		oldPopSize = theEngine->getPopulationSize();
	}

	return true;
}

//Added by Martin
//This thread controls the genetic engine
UINT RunEngineThread(LPVOID pParam)
{
	cout << "Thread started." << endl;

	//PopulationProperties* pThePopPropsPage = (PopulationProperties*)pParam;

	for(int i = 0; i < PopulationProperties::thePopPropsPage->m_nGenerations; i++){
		PopulationProperties::thePopPropsPage->theEngine->getPopulation()->getDesign(PopulationProperties::thePopPropsPage->currentDesign)->erase();
		PopulationProperties::thePopPropsPage->theEngine->runPopulation(1);
		PopulationProperties::thePopPropsPage->updateDialogText();
		PopulationProperties::thePopPropsPage->updateBreedingText();
		PopulationProperties::thePopPropsPage->interpret();
		SendMessage(PopulationProperties::thePopPropsPage->m_hWnd, ON_UPDATE_MESS, REFRESH, 0);
		if(bBreak)	break;
	}
	PostMessage(PopulationProperties::thePopPropsPage->m_hWnd, ON_UPDATE_MESS, RESTORE_BUTTONS, 0);
	return 0;
}

void PopulationProperties::OnRunNGenerations() 
{
	ostringstream message;
	if(!_taskUnderway){
		if (UpdateData(TRUE)) {

			message << "Running " << m_nGenerations << " generation";
			if (m_nGenerations > 1) message << "s...";
			else message << "...";
			newTask(message.str(), theEngine->getPopulationSize() * m_nGenerations);
			
			bBreak = false;
			((CButton*)GetDlgItem(IDC_RUN_N_GENERATIONS))->SetWindowText("Stop");
			GetDlgItem(IDC_SAVE_POPULATION)->EnableWindow(false);
			GetDlgItem(IDC_STARTGP)->EnableWindow(false);
			GetDlgItem(IDC_N_GENERATIONS_EDIT)->EnableWindow(false);
			GetDlgItem(IDC_SELECT_INDIVIDUAL)->EnableWindow(false);
			GetDlgItem(IDC_SHIFT_INDIVIDUAL)->EnableWindow(false);
			GetDlgItem(IDC_MODIFY_INDIVIDUAL_BUTTON)->EnableWindow(false);
			GetDlgItem(IDC_SAVE_INDIVIDUAL)->EnableWindow(false);
			
			AfxBeginThread(RunEngineThread, thePopPropsPage, THREAD_PRIORITY_BELOW_NORMAL);
			/*
			for (int i = 0; i < m_nGenerations; i++) {
				//remove all the layers
				theEngine->getPopulation()->getDesign(currentDesign)->erase();
				theEngine->runPopulation(1);
				updateDialogText();
				updateBreedingText();
				interpret();
			}
			taskFinished();
			*/
		}
	}
	else	bBreak = true;
}

LRESULT PopulationProperties::OnUpdateMess( WPARAM wParam, LPARAM lParam )
{
	switch(wParam)
	{
	case REFRESH:
		eval("refresh");
		break;
	case RESTORE_BUTTONS:
		taskFinished();
		GetDlgItem(IDC_SAVE_POPULATION)->EnableWindow(true);
		GetDlgItem(IDC_STARTGP)->EnableWindow(true);
		GetDlgItem(IDC_N_GENERATIONS_EDIT)->EnableWindow(true);
		GetDlgItem(IDC_SELECT_INDIVIDUAL)->EnableWindow(true);
		GetDlgItem(IDC_SHIFT_INDIVIDUAL)->EnableWindow(true);
		GetDlgItem(IDC_MODIFY_INDIVIDUAL_BUTTON)->EnableWindow(true);
		GetDlgItem(IDC_SAVE_INDIVIDUAL)->EnableWindow(true);
		((CButton*)GetDlgItem(IDC_RUN_N_GENERATIONS))->SetWindowText("Run");
		break;
	//case ERASE:

	}
	return 1;
}

void PopulationProperties::OnModifyIndividualButton() 
{
	theEngine->getPopulation()->getDesign(currentDesign)->erase();
	theEngine->getPopulation()->getDesign(currentDesign)->interpret();
	//setLayerVisibility(getCurrentDesign()->getPhenome().getLayerName(), true);
	//setCurrentLayer(getCurrentDesign()->getPhenome().getLayerName());
	//eval("refresh");
	ModifyIndividualDialog	modifyDialog;
	modifyDialog.DoModal();
	updateDialogText();
}


Design* PopulationProperties::getCurrentDesign()
{
	return theEngine->getPopulation()->getDesign(currentDesign);
}

void	PopulationProperties::newTask(string taskName, int length)
{
	((CStatic*)GetDlgItem(IDC_CURRENT_TASK))->SetWindowText(taskName.c_str());
	CProgressCtrl* bar = (CProgressCtrl*)GetDlgItem(IDC_TASK_PROGRESS);
	bar->SetRange32(0, length);
	bar->SetStep(1);
	bar->SetPos(0);
	_taskUnderway = true;
}

void	PopulationProperties::step(int steps)
{
	if (_taskUnderway)
		((CProgressCtrl*)GetDlgItem(IDC_TASK_PROGRESS))->OffsetPos(steps);
}

void	PopulationProperties::taskFinished()
{
	newTask("Idle...", 1);
	_taskUnderway = false;
}

bool PopulationProperties::taskUnderway()
{
	return _taskUnderway;
}

void PopulationProperties::OnSaveIndividual() 
{
	ostringstream layerName;

	layerName << "Gen" << theEngine->getGenerationsRun() << "Ind" << currentDesign + 1;
	string tempStr = layerName.str();
	getCurrentDesign()->saveDesign(tempStr);
	
}

void PopulationProperties::OnSavePopulation() 
{
	int popSize = theEngine->getPopulationSize();

	ostringstream layerName;
	string tempStr;
	newTask("Saving entire population...", popSize);
	for (int i = 0; i < popSize; i++) {
		layerName.str("");
		layerName << "Gen" << theEngine->getGenerationsRun() << "Ind" << i + 1;
		tempStr = layerName.str();
		theEngine->getPopulation()->getDesign(i)->saveDesign(tempStr);
		step();
	}
	taskFinished();
}

void PopulationProperties::reportSettings()
{
	Settings::settings->setSetting("PopulationProperties", "NGenerations", m_nGenerations);
}

void PopulationProperties::receiveSettings()
{

	int tempInt;
	if (Settings::settings->getSetting("PopulationProperties", "NGenerations", tempInt)) {
		m_nGenerations = tempInt;
	}
	else m_nGenerations = 1;

}


void PopulationProperties::reInterpret(bool entirePop)
{
	if(entirePop)	{
		Sleep(1);
	}
	else	{
		theEngine->getPopulation()->getDesign(currentDesign)->interpret(false);
	}
}
