// Ring.h: interface for the Ring class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RING_H__B350BB98_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
#define AFX_RING_H__B350BB98_E931_11D3_9F59_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <list>
#include <assert.h>

template <class T>
class RingIter;

template < class T >
class Ring {

  friend class RingIter<T>;
  
private:

  list<T> myBuffer;
  int myBegin, mySize;
  
public:
  typedef RingIter<T> iterator;
  typedef T value_type;
  typedef ptrdiff_t difference_type;
  typedef size_t size_type;
  typedef T * pointer;
  typedef T & reference;
  
  Ring() : myBegin( 0 ), mySize( 0 ) {}
  
  int size() const { return mySize; }
  
  void push_back( const T & value );
  void pop_front();
  T & front() const;
  T & back() const;
  
  iterator begin() { return iterator( *this, 0 ); }
  iterator end() { return iterator( *this, mySize ); }

private:
  T & operator[]( int i ) { return myBuffer[ index( i ) ]; }

  int index( int i ) const { return ( myBegin + i ) % N; }

};

template < class T, int N >
void Ring<T, N>::push_back( const T & value )
{
   myBuffer[ index( mySize ) ] = value;
   if ( full() )
     myBegin = index( 1 );
   else
     ++mySize;
}

template < class T, int N >
void Ring<T, N>::pop_front()
{
   assert( mySize > 0 );
   myBegin = index( 1 );
   --mySize;
}
   
template < class T, int N >
T & Ring<T, N>::front() const
{
   assert( mySize > 0 );
   return myBuffer[ myBegin ];
}
   
template < class T, int N >
T & Ring<T, N>::back() const
{
   assert( mySize > 0 );
   return myBuffer[ index( mySize - 1 ) ];
}
   


// Here's the iterator class for ring queue's.
//
// An iterator has
//
//   - a reference to a ring queue
//   - a current index into the ring queue

template < class T, int N >
class RingIter {
private:
  typedef RingIter<T, N> iterator;
  typedef Ring<T, N> container_type;
  typedef T value_type;

  container_type & myRing;
  int myOffset;
public:

  RingIter( container_type & Ring, int increment )
   : myRing( Ring ), myOffset( increment )
  {}

  bool operator==( const iterator & ri ) const
  {
     return ( &myRing == &ri.myRing
             && myOffset == ri.myOffset );
  }



  T & operator*() { return myRing[ myOffset ]; }
  
  iterator & operator++() { ++myOffset; return *this; }
  
  iterator operator++ ( int )
  { 
    RingIter<T, N> clone( *this );
    ++myOffset;
    return clone;
  }

};

template < class T, int N >
bool operator==( const RingIter<T, N> & ri1, const RingIter<T, N> & ri2)
{
  return ( ri1.myRing == ri2.myRing
           && ri1.myOffset == ri2.myOffset );
}

template < class T, int N >
bool operator!=( const RingIter<T, N> & ri1, const RingIter<T, N> & ri2)
{
  return ( ! (ri1 == ri2) );
}

#endif // !defined(AFX_RING_H__B350BB98_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
