//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Shape.h: interface for the Shape class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SHAPE_H__B350BB9D_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
#define AFX_SHAPE_H__B350BB9D_E931_11D3_9F59_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PEType.h"
#include "GeneticUtility.h"
#include "Design.h"
#include "MayaInfo.h"


typedef enum {UPPER, LOWER} UpperLower; 


class Shape  
{
public:
	Shape();
	Shape(const Shape& oldShape);
	Shape(MayaInfo* MI);
	virtual ~Shape();

// Accessors

	string&	getName() {return mayaShape->name();};
	string& getLoftName() {return loftName;};
	PEType& getPEType() {return assignment;};
	const PEType& getPETypeConst() const {return assignment;};
	double getZWidening() const {return ZWidening;};
	double getZHeight() const {return ZHeight;};
	double getZStart() const {return ZStart;};
	int getShapeMode() const {return shapeMode;};

	virtual bool isPrimary() {return false;};	// Not a primary shape
	
// Mutators

	void setMayaName(string& newName);
	void setPEType(PEType newType) {assignment = newType;};
	void setZWidening(double newWidening) {ZWidening = newWidening;};
	void setShapeMode(int newMode) {shapeMode = newMode;};
	Shape* ZCut(double loc, UpperLower become, bool startFromZero = FALSE);		// Cuts shape in two at Z = loc, this shape becomes upper or lower depending on value of "become"; the other is returned

	virtual void reduceNumZoneTypes(int newNumZoneTypes);

	void setZHeight(double height)	{ZHeight = height;};

// Maya-related functions

	void setEraseOnDelete(bool newErase) {eraseOnDelete = newErase;};
	void erase(); // erase and clean up Maya objects
	bool loft(bool startFromZero);  // This version of loft keeps track of all the new shapes created during the lofting and adds them to the list
	virtual bool initializeInterpret() = 0;
	bool duplicate();	// Duplicates the Maya curve
	void cut(double start, double stop); // cut an opening in the shape
	void MayaSelect();

// Predicates
	bool Zintersects(Shape* B, bool startFromZero = false);
	bool intersects(Shape* B, bool startFromZero = false);

// Utility
	virtual Shape* fullDuplicate() = 0;		// Returns a copy of this shape and attaches it to a duplicated Maya curve

protected:
	PEType assignment;		// These track the current values of the shape as it is operated upon
	double ZWidening;
	int shapeMode;
	double ZHeight, ZStart, ZStartScale;
	bool	hasCut;
	double	cutStart, cutStop;

	MayaInfo*	mayaShape;		// This shape's current Maya info
	std::list<std::string> cleanupList;
	bool	eraseOnDelete;		// Erase Maya shapes when the shape is deleted? (Doesn't happen on plugin exit.)
	string	loftName;			// Maya name of the lofted surface
	
public:
	static	int _intersectionCheck;
};

#endif // !defined(AFX_SHAPE_H__B350BB9D_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
