//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// ZoneTypeProperties.cpp : implementation file
//

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "ZoneTypeProperties.h"
#include "ZoneTypes.h"
#include "GeneticEngine.h"
#include <sstream>
#include <iostream>
#include "Layer.h"
#include "Shape.h"
#include "PrimaryShape.h"
#include "PopulationProperties.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// ZoneTypeProperties property page

IMPLEMENT_DYNCREATE(ZoneTypeProperties, CPropertyPage)

ZoneTypeProperties::ZoneTypeProperties() : CPropertyPage(ZoneTypeProperties::IDD)
{
	//{{AFX_DATA_INIT(ZoneTypeProperties)
	m_zoneName = _T("");
	m_shaderName = _T("");
	m_MutationRate = 0;
	m_noMutations = FALSE;
	//}}AFX_DATA_INIT
}

ZoneTypeProperties::~ZoneTypeProperties()
{
}

void ZoneTypeProperties::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(ZoneTypeProperties)
	DDX_Text(pDX, IDC_ZONE_TYPE_NAME, m_zoneName);
	DDX_CBString(pDX, IDC_ZONE_TYPE_SHADER, m_shaderName);
	DDX_Slider(pDX, IDC_MUTATIONRATE, m_MutationRate);
	DDX_Check(pDX, IDC_IN_OUT_MUTATIONS, m_noMutations);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(ZoneTypeProperties, CPropertyPage)
	//{{AFX_MSG_MAP(ZoneTypeProperties)
	ON_BN_CLICKED(IDC_ADD_NEW_ZONE_TYPE, OnAddNewZoneType)
	ON_BN_CLICKED(IDC_DELETE_ZONE_TYPE, OnDeleteZoneType)
	ON_LBN_SELCHANGE(IDC_ZONE_TYPE_LIST, OnSelchangeZoneTypeList)
	ON_EN_KILLFOCUS(IDC_ZONE_TYPE_NAME, OnKillfocusZoneTypeName)
	ON_CBN_SELENDOK(IDC_ZONE_TYPE_SHADER, OnSelendokZoneTypeShader)
	ON_BN_CLICKED(IDC_IN_OUT_MUTATIONS, OnInOutMutations)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_MUTATIONRATE, OnReleasedcaptureMutationrate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ZoneTypeProperties message handlers

void ZoneTypeProperties::OnAddNewZoneType() 
{
	if(PopulationProperties::thePopPropsPage->isInitialized())
		GeneticEngine::geneticEngine->makeAllImpressionsInvalid();

	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);

	int oldSel = zoneList->GetCurSel();
	ZoneTypes::zoneTypes->addZoneType("NewZoneType", *(ZoneTypes::zoneTypes->getShaderGroups().begin()),  oldSel);
	
	resetZoneList();
	if (oldSel < zoneList->GetCount() - 1) oldSel++;
	zoneList->SetCurSel(oldSel);
	OnSelchangeZoneTypeList();
	GotoDlgCtrl(GetDlgItem(IDC_ZONE_TYPE_NAME));
}

void ZoneTypeProperties::OnDeleteZoneType() 
{
	if(PopulationProperties::thePopPropsPage->isInitialized())
		GeneticEngine::geneticEngine->makeAllImpressionsInvalid();
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	int oldSel = zoneList->GetCurSel();
	ZoneTypes::zoneTypes->deleteZoneType(oldSel);
	cout << "ZoneType deleted" << endl;
	resetZoneList();
	if (oldSel >= zoneList->GetCount()) oldSel--;
	zoneList->SetCurSel(oldSel);
	OnSelchangeZoneTypeList();
}

void ZoneTypeProperties::OnSelchangeZoneTypeList() 
{
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	CComboBox* shaderList = (CComboBox*)GetDlgItem(IDC_ZONE_TYPE_SHADER);

	m_zoneName = ZoneTypes::zoneTypes->getZoneName(zoneList->GetCurSel()).c_str();
	m_noMutations = ZoneTypes::zoneTypes->getMutableZoneType(zoneList->GetCurSel());
	m_MutationRate = ZoneTypes::zoneTypes->getMutationRate(zoneList->GetCurSel());
	string muttext = "Mutation rate for " + m_zoneName;
	((CStatic*)GetDlgItem(IDC_MUTATION_TEXT))->SetWindowText(muttext.c_str());
	UpdateData(false);

	m_shaderName = ZoneTypes::zoneTypes->getZoneShader(zoneList->GetCurSel()).c_str();

	if (! ZoneTypes::zoneTypes->validShader(m_shaderName.GetBuffer(m_shaderName.GetLength())))
	{
		shaderList->SetCurSel(0);
		UpdateData(true);
	}
	else
		UpdateData(false);

}

void ZoneTypeProperties::resetZoneList()
{
	ostringstream tempString;

	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	zoneList->ResetContent();

	for (int i = 0; i < ZoneTypes::zoneTypes->getNumZoneTypes(); i++) {
		tempString.str("");
		tempString << ZoneTypes::zoneTypes->getZoneName(i);
		tempString << '\t';
		if (ZoneTypes::zoneTypes->validShader(ZoneTypes::zoneTypes->getZoneShader(i))) {
			tempString << ZoneTypes::zoneTypes->getZoneShader(i);
		}
		else {
			ZoneTypes::zoneTypes->setZoneShader("~", i);
			tempString << "     --";
		}
		zoneList->AddString(tempString.str().c_str());
	}

	if (zoneList->GetCount() <= 1)
		(CButton*)GetDlgItem(IDC_DELETE_ZONE_TYPE)->EnableWindow(false);
	else (CButton*)GetDlgItem(IDC_DELETE_ZONE_TYPE)->EnableWindow(true);
}

void ZoneTypeProperties::OnKillfocusZoneTypeName() 
{
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	int zoneNum = zoneList->GetCurSel();

	UpdateData(true);
	ZoneTypes::zoneTypes->setZoneName(m_zoneName.GetBuffer(m_zoneName.GetLength()), zoneNum);
	resetZoneList();
	zoneList->SetCurSel(zoneNum);
	//Check if there is a layer with the same name
	ZoneTypes::zoneTypes->checkLayerNames();
}

void ZoneTypeProperties::OnSelendokZoneTypeShader() 
{
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	int zoneNum = zoneList->GetCurSel();

	UpdateData(true);
	ZoneTypes::zoneTypes->setZoneShader(m_shaderName.GetBuffer(m_shaderName.GetLength()), zoneNum);
	resetZoneList();
	zoneList->SetCurSel(zoneNum);	
}

BOOL ZoneTypeProperties::OnSetActive() 
{
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);

	resetZoneList();
	
	zoneList->SetCurSel(0);
	OnSelchangeZoneTypeList();
	return CPropertyPage::OnSetActive();
}

BOOL ZoneTypeProperties::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	int tabStop = 90;
	zoneList->SetTabStops( 1, &tabStop );

	CComboBox* shaderList = (CComboBox*)GetDlgItem(IDC_ZONE_TYPE_SHADER);
	shaderList->ResetContent();
	for (list<string>::iterator i = ZoneTypes::zoneTypes->getShaderGroups().begin(); i != ZoneTypes::zoneTypes->getShaderGroups().end(); i++) {
		shaderList->AddString((*i).c_str());
	}
	shaderList->SetCurSel(0);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void ZoneTypeProperties::OnInOutMutations() 
{
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	int zoneNum = zoneList->GetCurSel();

	UpdateData(true);
	ZoneTypes::zoneTypes->setMutableZoneType(zoneNum, m_noMutations);

}


void ZoneTypeProperties::OnReleasedcaptureMutationrate(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	CListBox* zoneList = (CListBox*)GetDlgItem(IDC_ZONE_TYPE_LIST);
	int zoneNum = zoneList->GetCurSel();

	ZoneTypes::zoneTypes->setMutationRate(zoneNum, m_MutationRate);
	*pResult = 0;
}

